# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
"""
This module contains the mapping between gRPC calls and Python/dazl types.
"""

from __future__ import annotations

import asyncio
from asyncio import wait_for
import contextlib
from datetime import datetime, timedelta
import sys
from typing import (
    AbstractSet,
    Any,
    AsyncIterable,
    Collection,
    ContextManager,
    Iterator,
    Mapping,
    Optional,
    Sequence,
    Union,
    overload,
)
import warnings

from grpc import ChannelConnectivity
from grpc.aio import Channel, UnaryStreamCall, UsageError

from .. import aio
from ... import LOG
from ..._gen.com.daml.ledger.api import v1 as lapipb
from ..._gen.com.daml.ledger.api.v1 import admin as lapiadminpb
from ...damlast.daml_lf_1 import PackageRef, TypeConName
from ...damlast.util import is_match
from ...prim import (
    ContractData,
    ContractId,
    Parties,
    Party,
    TimeDeltaLike,
    datetime_to_timestamp,
    to_parties,
    to_timedelta,
)
from ...query import Filter, Queries, Query, parse_query
from .._call import (
    AnyCallParameters,
    CachedParameters,
    CallParameters,
    ConnectionParameters,
    ReadCallParameters,
    WriteCallParameters,
)
from .._offsets import END, End, LedgerOffsetRange, from_offset_until_forever
from .._retry import retry
from ..api_types import (
    ArchiveEvent,
    Boundary,
    CommandMeta,
    Commands,
    CreateEvent,
    ExerciseResponse,
    MeteringReport,
    PartyInfo,
    Right,
    User,
    Version,
    to_commands,
)
from ..auth import TokenOrTokenProvider
from ..config import Config
from ..errors import ProtocolWarning, _allow_cancel, _translate_exceptions
from ._args_aio import CallContext
from .channel import create_channel
from .codec_aio import Codec

if sys.version_info >= (3, 11):
    from typing import Unpack
else:
    from typing_extensions import Unpack

__all__ = ["Connection", "QueryStream"]


class Connection(aio.Connection):
    """
    An asynchronous (``asyncio``) connection to the Daml gRPC Ledger API.
    """

    def __init__(self, config: Config):
        self._config = config
        self._logger = config.logger
        self._channel = create_channel(config)
        self._codec = Codec(self, lookup=config.lookup)
        self._cache: Optional[CachedParameters] = None

    @property
    def config(self) -> Config:
        return self._config

    @property
    def channel(self) -> Channel:
        """
        Provides access to the underlying gRPC channel.
        """
        return self._channel

    @property
    def codec(self) -> Codec:
        return self._codec

    @property
    def is_closed(self) -> bool:
        return self._channel.get_state(try_to_connect=False) == ChannelConnectivity.SHUTDOWN

    async def __aenter__(self) -> Connection:
        await self.open()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        await self.close()

    async def open(self) -> None:
        """
        Does nothing, and is only supplied for symmetry with :meth:`close`.
        """

    async def close(self) -> None:
        """
        Close the underlying channel. Once the channel is closed, future command submissions,
        streams in progress, and any future streams will fail.
        """
        # TODO: grpc-stubs have a bug here; close does not require a parameter
        await self._channel.close()  # type: ignore

    def _retry_timeout(self, timeout: Optional[TimeDeltaLike]) -> timedelta:
        return to_timedelta(timeout) if timeout is not None else self._config.url.retry_timeout

    # region Write API

    @overload
    def _call(self, **kwargs: Unpack[CallParameters]) -> ContextManager[CallContext]: ...

    @overload
    def _call(self, **kwargs: Unpack[ReadCallParameters]) -> ContextManager[CallContext]: ...

    @overload
    def _call(self, **kwargs: Unpack[WriteCallParameters]) -> ContextManager[CallContext]: ...

    @contextlib.contextmanager
    def _call(self, **call_kwargs: Unpack[AnyCallParameters]) -> Iterator[CallContext]:
        connection_params = ConnectionParameters(
            read_as=to_parties(self._config.access.read_as),
            act_as=to_parties(self._config.access.act_as),
            ledger_id=self._config.access.ledger_id,
            user_id_or_application_name=self._config.access.user_id
            or self._config.access.application_name,
            token=self._config.access.token,
            timeout=self._config.url.retry_timeout,
        )
        call = CallContext.compute(call_kwargs, self._cache, connection_params)
        call.channel = self.channel
        yield call
        self._cache = call.get_cache()

    async def submit(
        self,
        commands: Commands,
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Optional[Parties] = None,
        act_as: Optional[Parties] = None,
        application_name: Optional[str] = None,
        user_id: Optional[str] = None,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> None:
        """
        Submit one or more commands to the Ledger API.

        You should generally prefer trying to use :meth:`create`, :meth:`exercise`,
        :meth:`exercise_by_key`, or :meth:`create_and_exercise`, as they are available over both
        the gRPC Ledger API and HTTP JSON API; additionally those methods can provide more
        information about what happened.

        This method can be used to submit multiple disparate commands as a single transaction, but
        if you find yourself needing to do this, you may want to consider moving more of your logic
        into Daml so that only a single command is needed from the outside in order to satisfy your
        use case.
        """
        with self._call(
            workflow_id=workflow_id,
            command_id=command_id,
            read_as=read_as,
            act_as=act_as,
            application_name=application_name,
            user_id=user_id,
            token=token,
            timeout=timeout,
        ) as call:
            commands = to_commands(commands)
            if not commands:
                return

            if len(commands) == 1:
                command_seq = [await self._codec.encode_command(commands[0], token=token)]
            else:
                command_seq = await asyncio.gather(
                    *(self._codec.encode_command(command, token=token) for command in commands)
                )

            stub = call.grpc_stub(lapipb.CommandServiceStub)
            request = self._submit_and_wait_request(command_seq, await call.command_meta())
            await retry(
                lambda: stub.SubmitAndWait(request, **call.grpc_kwargs),
                timeout=call.timeout,
            )

    async def create(
        self,
        template_id: Union[str, TypeConName],
        payload: ContractData,
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Optional[Parties] = None,
        act_as: Optional[Parties] = None,
        application_name: Optional[str] = None,
        user_id: Optional[str] = None,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> CreateEvent:
        """
        Create a contract for a given template.

        :param template_id:
            The template of the contract to be created.
        :param payload:
            Template arguments for the contract to be created.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :param token:
            An optional authorization token. If unspecified, the one specified at connection
            creation time is used.
        :param read_as:
            An optional set of read-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param act_as:
            An optional set of act-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param application_name:
            An optional application name to use instead of the one configured at connection
            creation time. Cannot be specified with ``user_id``.
        :param user_id:
            An optional user ID to use instead of the one configured at connection creation time
            or from the ``sub`` claim of the ``token``. Cannot be specified with
            ``application_name``.
        :param timeout:
            The length of time to wait before giving up on this submission. If unspecified, the
            configuration value of ``retry_timeout`` is used.
        :return:
            The :class:`CreateEvent` that represents the contract that was successfully created.
        """
        with self._call(
            workflow_id=workflow_id,
            command_id=command_id,
            read_as=read_as,
            act_as=act_as,
            application_name=application_name,
            user_id=user_id,
            token=token,
            timeout=timeout,
        ) as call:
            commands = [
                lapipb.Command(
                    create=await self._codec.encode_create_command(
                        template_id, payload, token=token
                    )
                )
            ]

            stub = call.grpc_stub(lapipb.CommandServiceStub)
            request = self._submit_and_wait_request(commands, await call.command_meta())
            response = await retry(
                lambda: stub.SubmitAndWaitForTransaction(request, **call.grpc_kwargs),
                timeout=call.timeout,
            )
            return await self._codec.decode_created_event(
                response.transaction.events[0].created, token=token
            )

    async def exercise(
        self,
        contract_id: ContractId,
        choice_name: str,
        argument: Optional[ContractData] = None,
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Optional[Parties] = None,
        act_as: Optional[Parties] = None,
        application_name: Optional[str] = None,
        user_id: Optional[str] = None,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> ExerciseResponse:
        """
        Exercise a choice on a contract identified by its contract ID.

        :param contract_id:
            The contract ID of the contract to exercise.
        :param choice_name:
            The name of the choice to exercise.
        :param argument:
            The choice arguments. Can be omitted for choices that take no argument.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :param read_as:
            An optional set of read-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param act_as:
            An optional set of act-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param application_name:
            An optional application name to use instead of the one configured at connection
            creation time. Cannot be specified with ``user_id``.
        :param user_id:
            An optional user ID to use instead of the one configured at connection creation time
            or from the ``sub`` claim of the ``token``. Cannot be specified with
            ``application_name``.
        :param token:
            An optional token string, or function/async function that returns a token. If provided,
            this overrides the token passed in at connection construction time.
        :param timeout:
            The length of time to wait before giving up on this submission. If unspecified, the
            configuration value of ``retry_timeout`` is used.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        with self._call(
            workflow_id=workflow_id,
            command_id=command_id,
            read_as=read_as,
            act_as=act_as,
            application_name=application_name,
            user_id=user_id,
            token=token,
            timeout=timeout,
        ) as call:
            commands = [
                lapipb.Command(
                    exercise=await self._codec.encode_exercise_command(
                        contract_id, choice_name, argument, token=call.token
                    )
                )
            ]

            stub = call.grpc_stub(lapipb.CommandServiceStub)
            request = self._submit_and_wait_request(commands, await call.command_meta())
            response = await retry(
                lambda: stub.SubmitAndWaitForTransactionTree(request, **call.grpc_kwargs),
                timeout=call.timeout,
            )
            return await self._codec.decode_exercise_response(
                response.transaction, token=call.token
            )

    async def create_and_exercise(
        self,
        template_id: Union[str, TypeConName],
        payload: ContractData,
        choice_name: str,
        argument: Optional[ContractData] = None,
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Optional[Parties] = None,
        act_as: Optional[Parties] = None,
        application_name: Optional[str] = None,
        user_id: Optional[str] = None,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> ExerciseResponse:
        """
        Exercise a choice on a newly-created contract, in a single transaction.

        :param template_id:
            The template of the contract to be created (positional argument only).
        :param payload:
            Template arguments for the contract to be created (positional argument only).
        :param choice_name:
            The name of the choice to exercise (positional argument only).
        :param argument:
            The choice arguments. Can be omitted for choices that take no argument (positional
            argument only).
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :param read_as:
            An optional set of read-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param act_as:
            An optional set of act-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param application_name:
            An optional application name to use instead of the one configured at connection
            creation time. Cannot be specified with ``user_id``.
        :param user_id:
            An optional user ID to use instead of the one configured at connection creation time
            or from the ``sub`` claim of the ``token``. Cannot be specified with
            ``application_name``.
        :param token:
            An optional token string, or function/async function that returns a token. If provided,
            this overrides the token passed in at connection construction time.
        :param timeout:
            The length of time to wait before giving up on this submission. If unspecified, the
            configuration value of ``retry_timeout`` is used.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        with self._call(
            workflow_id=workflow_id,
            command_id=command_id,
            read_as=read_as,
            act_as=act_as,
            application_name=application_name,
            user_id=user_id,
            token=token,
            timeout=timeout,
        ) as call:
            commands = [
                lapipb.Command(
                    createAndExercise=await self._codec.encode_create_and_exercise_command(
                        template_id, payload, choice_name, argument, token=call.token
                    )
                )
            ]

            stub = call.grpc_stub(lapipb.CommandServiceStub)
            request = self._submit_and_wait_request(commands, await call.command_meta())
            response = await retry(
                lambda: stub.SubmitAndWaitForTransactionTree(request, **call.grpc_kwargs),
                timeout=call.timeout,
            )
            return await self._codec.decode_exercise_response(
                response.transaction, token=call.token
            )

    async def exercise_by_key(
        self,
        template_id: Union[str, TypeConName],
        choice_name: str,
        key: Any,
        argument: Optional[ContractData] = None,
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Optional[Parties] = None,
        act_as: Optional[Parties] = None,
        application_name: Optional[str] = None,
        user_id: Optional[str] = None,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> ExerciseResponse:
        """
        Exercise a choice on a contract identified by its contract key.

        :param template_id:
            The template of the contract to be created (positional argument only).
        :param choice_name:
            The name of the choice to exercise.
        :param key:
            The key of the contract to exercise.
        :param argument:
            The choice arguments. Can be omitted for choices that take no argument.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :param read_as:
            An optional set of read-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param act_as:
            An optional set of act-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param application_name:
            An optional application name to use instead of the one configured at connection
            creation time. Cannot be specified with ``user_id``.
        :param user_id:
            An optional user ID to use instead of the one configured at connection creation time
            or from the ``sub`` claim of the ``token``. Cannot be specified with
            ``application_name``.
        :param token:
            An optional token string, or function/async function that returns a token. If provided,
            this overrides the token passed in at connection construction time.
        :param timeout:
            The length of time to wait before giving up on this submission. If unspecified, the
            configuration value of ``retry_timeout`` is used.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        with self._call(
            workflow_id=workflow_id,
            command_id=command_id,
            read_as=read_as,
            act_as=act_as,
            application_name=application_name,
            user_id=user_id,
            token=token,
            timeout=timeout,
        ) as call:
            commands = [
                lapipb.Command(
                    exerciseByKey=await self._codec.encode_exercise_by_key_command(
                        template_id, choice_name, key, argument, token=call.token
                    )
                )
            ]

            stub = call.grpc_stub(lapipb.CommandServiceStub)
            request = self._submit_and_wait_request(commands, await call.command_meta())
            response = await stub.SubmitAndWaitForTransactionTree(request, **call.grpc_kwargs)

            return await self._codec.decode_exercise_response(
                response.transaction, token=call.token
            )

    async def archive(
        self,
        contract_id: ContractId,
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Optional[Parties] = None,
        act_as: Optional[Parties] = None,
        application_name: Optional[str] = None,
        user_id: Optional[str] = None,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> ArchiveEvent:
        """
        Archive a choice on a contract identified by its contract ID.

        :param contract_id:
            The contract ID of the contract to exercise.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :param read_as:
            An optional set of read-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param act_as:
            An optional set of act-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param application_name:
            An optional application name to use instead of the one configured at connection
            creation time. Cannot be specified with ``user_id``.
        :param user_id:
            An optional user ID to use instead of the one configured at connection creation time
            or from the ``sub`` claim of the ``token``. Cannot be specified with
            ``application_name``.
        :param token:
            An optional token string, or function/async function that returns a token. If provided,
            this overrides the token passed in at connection construction time.
        :param timeout:
            The length of time to wait before giving up on this submission. If unspecified, the
            configuration value of ``retry_timeout`` is used.
        :return:
            The :class:`ArchiveEvent` that indicates this contract was archived.
        """
        await self.exercise(
            contract_id,
            "Archive",
            workflow_id=workflow_id,
            command_id=command_id,
            read_as=read_as,
            act_as=act_as,
            application_name=application_name,
            user_id=user_id,
            token=token,
            timeout=timeout,
        )
        return ArchiveEvent(contract_id)

    async def archive_by_key(
        self,
        template_id: str,
        key: Any,
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Optional[Parties] = None,
        act_as: Optional[Parties] = None,
        application_name: Optional[str] = None,
        user_id: Optional[str] = None,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> ArchiveEvent:
        """
        Exercise a choice on a contract identified by its contract key.

        :param template_id:
            The template of the contract to be created (positional argument only).
        :param key:
            The key of the contract to exercise.
            time; it should be passed in as a positional parameter and never by name.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :param read_as:
            An optional set of read-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param act_as:
            An optional set of act-as parties to use to submit this command. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param application_name:
            An optional application name to use instead of the one configured at connection
            creation time. Cannot be specified with ``user_id``.
        :param user_id:
            An optional user ID to use instead of the one configured at connection creation time
            or from the ``sub`` claim of the ``token``. Cannot be specified with
            ``application_name``.
        :param token:
            An optional token string, or function/async function that returns a token. If provided,
            this overrides the token passed in at connection construction time.
        :param timeout:
            The length of time to wait before giving up on this submission. If unspecified, the
            configuration value of ``retry_timeout`` is used.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        response = await self.exercise_by_key(
            template_id,
            "Archive",
            key,
            workflow_id=workflow_id,
            command_id=command_id,
            read_as=read_as,
            act_as=act_as,
            application_name=application_name,
            user_id=user_id,
            token=token,
            timeout=timeout,
        )
        return next(iter(event for event in response.events if isinstance(event, ArchiveEvent)))

    def _submit_and_wait_request(
        self,
        commands: Collection[lapipb.Command],
        meta: CommandMeta,
        /,
        ledger_id: Optional[str] = None,
    ) -> lapipb.SubmitAndWaitRequest:
        return lapipb.SubmitAndWaitRequest(
            commands=lapipb.Commands(
                ledger_id=ledger_id,
                application_id=meta.application_name,
                command_id=meta.command_id,
                workflow_id=meta.workflow_id,
                commands=commands,
                act_as=meta.act_as,
                read_as=meta.read_as,
            )
        )

    # endregion

    # region Read API

    async def get_ledger_end(
        self,
        *,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> str:
        """
        Return the offset at the end of the ledger.

        :param token:
            An optional token string, or function/async function that returns a token. If provided,
            this overrides the token passed in at connection construction time.
        :param timeout:
            The length of time to wait before giving up on this submission. If unspecified, the
            configuration value of ``retry_timeout`` is used.
        """
        with self._call(token=token, timeout=timeout) as call:
            stub = call.grpc_stub(lapipb.TransactionServiceStub)
            request = lapipb.GetLedgerEndRequest(ledger_id=call.ledger_id)
            response = await retry(
                lambda: stub.GetLedgerEnd(request, **call.grpc_kwargs),
                timeout=call.timeout,
            )
            return response.offset.absolute

    def query(
        self,
        template_or_interface_id: Union[str, TypeConName] = "*",
        query: Query = None,
        /,
        *,
        begin_offset: Optional[str] = None,
        end_offset: Optional[str] = None,
        read_as: Optional[Parties] = None,
        user_id: Optional[str] = None,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> QueryStream:
        """
        Return the create events from the active contract set service as a stream.

        If you find yourself repeatedly calling :meth:`query` or :meth:`query_many` over the same
        set of templates, you may want to consider :class:`ACS` instead, which is a utility class
        that helps you maintain a "live" state of the ACS.

        :param template_or_interface_id:
            The name of the template or interface for which to fetch contracts.
        :param query:
            A filter to apply to the set of returned contracts.
        :param begin_offset:
            The starting offset at which to read an active contract set. If ``None``, contracts
            are read from the beginning, and using the Active Contract Set Service instead of the
            Transaction Service.
        :param end_offset:
            The ending offset. If ``None``, contracts are read until the end of the stream.
            In order to read indefinitely, use :meth:`stream` instead.
        :param read_as:
            An optional set of read-as parties to use to submit this query. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param user_id:
            An optional user ID to use instead of the one configured at connection creation time
            or from the ``sub`` claim of the ``token``.
        :param token:
            An optional token string, or function/async function that returns a token. If provided,
            this overrides the token passed in at connection construction time.
        :param timeout:
            The length of time to wait before giving up on a query. If unspecified, the
            configuration value of ``retry_timeout`` is used.
        """
        with self._call(read_as=read_as, user_id=user_id, token=token, timeout=timeout) as call:
            offset = LedgerOffsetRange(begin_offset, end_offset if end_offset is not None else END)
            return QueryStream(
                self,
                parse_query({template_or_interface_id: query}, server_side_filters=False),
                offset,
                call,
            )

    def query_many(
        self,
        *queries: Queries,
        begin_offset: Optional[str] = None,
        end_offset: Optional[str] = None,
        read_as: Optional[Parties] = None,
        user_id: Optional[str] = None,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> QueryStream:
        """
        Return the create events from the active contract set service as a stream.

        If you find yourself repeatedly calling :meth:`query` or :meth:`query_many` over the same
        set of templates, you may want to consider :class:`ACS` instead, which is a utility class
        that helps you maintain a "live" state of the ACS.

        :param queries:
            A map of template IDs to filter to apply to the set of returned contracts.
        :param begin_offset:
            The starting offset at which to read an active contract set. If ``None``, contracts
            are read from the beginning, and using the Active Contract Set Service instead of the
            Transaction Service.
        :param end_offset:
            The ending offset. If ``None``, contracts are read until the end of the stream.
            In order to read indefinitely, use :meth:`stream_many` instead.
        :param read_as:
            An optional set of read-as parties to use to submit this query. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param user_id:
            An optional user ID to use instead of the one configured at connection creation time
            or from the ``sub`` claim of the ``token``.
        :param token:
            An optional token string, or function/async function that returns a token. If provided,
            this overrides the token passed in at connection construction time.
        :param timeout:
            The length of time to wait before giving up on a query. If unspecified, the
            configuration value of ``retry_timeout`` is used.
        """
        with self._call(read_as=read_as, user_id=user_id, token=token, timeout=timeout) as call:
            offset = LedgerOffsetRange(begin_offset, end_offset if end_offset is not None else END)

            return QueryStream(
                self,
                parse_query(*queries, server_side_filters=False),
                offset,
                call,
            )

    def stream(
        self,
        template_or_interface_id: Union[str, TypeConName] = "*",
        query: Query = None,
        /,
        *,
        offset: Optional[str] = None,
        read_as: Optional[Parties] = None,
        user_id: Optional[str] = None,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> QueryStream:
        """
        Stream create/archive events.

        When ``offset`` is ``None``, create events from the active contract set are returned first,
        followed by a continuous stream of updates (creates/archives).

        Otherwise, ``offset`` can be supplied to resume a stream from a prior point where a
        ``Boundary`` was returned from a previous stream.

        :param template_or_interface_id:
            The name of the template or interface for which to fetch contracts.
        :param query:
            A filter to apply to the set of returned contracts. Note that this does not filter
            :class:`ArchiveEvent`; readers of the stream MUST be able to cope with "mismatched"
            archives that come from the result of applying a filter.
        :param offset:
            An optional offset at which to start receiving events. If ``None``, start from the
            beginning.
        :param read_as:
            An optional set of read-as parties to use to submit this query. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param user_id:
            An optional user ID to use instead of the one configured at connection creation time
            or from the ``sub`` claim of the ``token``.
        :param token:
            An optional token string, or function/async function that returns a token. If provided,
            this overrides the token passed in at connection construction time.
        :param timeout:
            The length of time to wait before giving up on a query. If unspecified, the
            configuration value of ``retry_timeout`` is used.
        """
        with self._call(read_as=read_as, user_id=user_id, token=token, timeout=timeout) as call:
            return QueryStream(
                self,
                parse_query({template_or_interface_id: query}, server_side_filters=False),
                from_offset_until_forever(offset),
                call,
            )

    def stream_many(
        self,
        *queries: Queries,
        offset: Optional[str] = None,
        read_as: Optional[Parties] = None,
        user_id: Optional[str] = None,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> QueryStream:
        """
        Stream create/archive events from more than one template ID in the same stream.

        When ``offset`` is ``None``, create events from the active contract set are returned first,
        followed by a continuous stream of updates (creates/archives).

        Otherwise, ``offset`` can be supplied to resume a stream from a prior point where a
        ``Boundary`` was returned from a previous stream.

        :param queries:
            A map of template IDs to filter to apply to the set of returned contracts. Note that
            this does not filter :class:`ArchiveEvent`; readers of the stream MUST be able to cope
            with "mismatched" archives that come from the result of applying a filter.
        :param offset:
            An optional offset at which to start receiving events. If ``None``, start from the
            beginning.
        :param read_as:
            An optional set of read-as parties to use to submit this query. Note that for a
            ledger with authorization, these parties must be a subset of the parties in the token.
        :param user_id:
            An optional user ID to use instead of the one configured at connection creation time
            or from the ``sub`` claim of the ``token``.
        :param token:
            An optional token string, or function/async function that returns a token. If provided,
            this overrides the token passed in at connection construction time.
        :param timeout:
            The length of time to wait before giving up on a query. If unspecified, the
            configuration value of ``retry_timeout`` is used.
        """
        with self._call(read_as=read_as, user_id=user_id, token=token, timeout=timeout) as call:
            return QueryStream(
                self,
                parse_query(*queries, server_side_filters=False),
                from_offset_until_forever(offset),
                call,
            )

    # endregion

    async def get_version(
        self,
        *,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> Version:
        with self._call(token=token, timeout=timeout) as call:
            stub = call.grpc_stub(lapipb.VersionServiceStub)
            request = lapipb.GetLedgerApiVersionRequest(ledger_id=call.ledger_id)

            response = await retry(
                lambda: stub.GetLedgerApiVersion(request, **call.grpc_kwargs), timeout=call.timeout
            )
            return Codec.decode_version(response)

    # region User Management calls

    async def get_user(
        self,
        user_id: Optional[str] = None,
        /,
        *,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> User:
        with self._call(token=token, timeout=timeout) as call:
            stub = call.grpc_stub(lapiadminpb.UserManagementServiceStub)
            request = lapiadminpb.GetUserRequest(user_id=user_id)
            response = await retry(
                lambda: stub.GetUser(request, **call.grpc_kwargs), timeout=call.timeout
            )
            return Codec.decode_user(response.user)

    async def create_user(
        self,
        user: User,
        rights: Optional[Sequence[Right]] = None,
        *,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> User:
        with self._call(token=token, timeout=timeout) as call:
            stub = call.grpc_stub(lapiadminpb.UserManagementServiceStub)
            request = lapiadminpb.CreateUserRequest(user=Codec.encode_user(user))
            if rights is not None:
                request.rights.extend(Codec.encode_right(right) for right in rights)
            response = await retry(
                lambda: stub.CreateUser(request, **call.grpc_kwargs), timeout=call.timeout
            )
            return Codec.decode_user(response.user)

    async def list_users(
        self,
        *,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> Sequence[User]:
        with self._call(token=token, timeout=timeout) as call:
            stub = call.grpc_stub(lapiadminpb.UserManagementServiceStub)
            request = lapiadminpb.ListUsersRequest()
            response = await retry(
                lambda: stub.ListUsers(request, **call.grpc_kwargs), timeout=call.timeout
            )
            return [Codec.decode_user(user) for user in response.users]

    async def list_user_rights(
        self,
        user_id: Optional[str] = None,
        *,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> Sequence[Right]:
        with self._call(token=token, timeout=timeout) as call:
            stub = call.grpc_stub(lapiadminpb.UserManagementServiceStub)
            request = lapiadminpb.ListUserRightsRequest(user_id=user_id)
            response = await retry(
                lambda: stub.ListUserRights(request, **call.grpc_kwargs), timeout=call.timeout
            )
            return [Codec.decode_right(user) for user in response.rights]

    # endregion

    # region Party Management calls

    async def allocate_party(
        self,
        *,
        identifier_hint: Optional[str] = None,
        display_name: Optional[str] = None,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> PartyInfo:
        """
        Allocate a new party.
        """
        with self._call(token=token, timeout=timeout) as call:
            stub = call.grpc_stub(lapiadminpb.PartyManagementServiceStub)
            request = lapiadminpb.AllocatePartyRequest(
                party_id_hint=Party(identifier_hint) if identifier_hint else None,
                display_name=display_name,
            )
            response = await retry(
                lambda: stub.AllocateParty(request, **call.grpc_kwargs), timeout=call.timeout
            )
            return Codec.decode_party_info(response.party_details)

    async def list_known_parties(
        self,
        *,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> Sequence[PartyInfo]:
        with self._call(token=token, timeout=timeout) as call:
            stub = call.grpc_stub(lapiadminpb.PartyManagementServiceStub)
            request = lapiadminpb.ListKnownPartiesRequest()
            response = await retry(
                lambda: stub.ListKnownParties(request, **call.grpc_kwargs), timeout=call.timeout
            )
            return [Codec.decode_party_info(pd) for pd in response.party_details]

    # endregion

    # region Package Management calls

    async def get_package(
        self,
        package_id: PackageRef,
        /,
        *,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> bytes:
        with self._call(token=token, timeout=timeout) as call:
            stub = call.grpc_stub(lapipb.PackageServiceStub)
            request = lapipb.GetPackageRequest(ledger_id=call.ledger_id, package_id=package_id)

            response = await retry(
                lambda: stub.GetPackage(request, **call.grpc_kwargs), timeout=call.timeout
            )
            return response.archive_payload

    async def list_package_ids(
        self,
        *,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> AbstractSet[PackageRef]:
        with self._call(token=token, timeout=timeout) as call:
            stub = call.grpc_stub(lapipb.PackageServiceStub)
            request = lapipb.ListPackagesRequest(ledger_id=call.ledger_id)
            response = await retry(
                lambda: stub.ListPackages(request, **call.grpc_kwargs), timeout=call.timeout
            )
            return frozenset({PackageRef(pkg_id) for pkg_id in response.package_ids})

    async def upload_package(
        self,
        contents: bytes,
        *,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> None:
        with self._call(token=token, timeout=timeout) as call:
            stub = call.grpc_stub(lapiadminpb.PackageManagementServiceStub)
            request = lapiadminpb.UploadDarFileRequest(dar_file=contents)
            await retry(
                lambda: stub.UploadDarFile(request, **call.grpc_kwargs), timeout=call.timeout
            )

            # assuming the package has been uploaded, go ahead and load that locally too
            await self._codec.preload(contents)

    # endregion

    # region Metering Report calls

    async def get_metering_report(
        self,
        from_: datetime,
        to: Optional[datetime] = None,
        application_id: Optional[str] = None,
        *,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> MeteringReport:
        with self._call(token=token, timeout=timeout) as call:
            stub = call.grpc_stub(lapiadminpb.MeteringReportServiceStub)

            # This slightly awkward construction is due to Protobuf's Python type not escaping field
            # names that happen to match keywords. Unfortunately mypy can't follow what is going on
            # when we do this, so we must be careful!
            kwargs = {"from": datetime_to_timestamp(from_)}  # type: dict[str, Any]
            if to is not None:
                kwargs["to"] = datetime_to_timestamp(to)
            if application_id is not None:
                kwargs["application_id"] = application_id
            request = lapiadminpb.GetMeteringReportRequest(**kwargs)  # type: ignore

            response = await retry(
                lambda: stub.GetMeteringReport(request, **call.grpc_kwargs), timeout=call.timeout
            )
            return Codec.decode_get_metering_report_response(response)

    # endregion


class QueryStream(aio.QueryStreamBase):
    def __init__(
        self,
        conn: Connection,
        filters: Optional[Mapping[TypeConName, Filter]],
        offset_range: LedgerOffsetRange,
        call: CallContext,
    ):
        self.conn = conn
        self._filters = filters
        self._offset_range = offset_range
        self._call = call
        self._response_stream = None  # type: Optional[UnaryStreamCall]
        self._closed = False

    @property
    def is_closed(self) -> bool:
        return self._closed

    async def close(self) -> None:
        # make sure to mark the object as "closed"; when we're closed, we don't mind cancellation
        # errors, because we're the one triggering the cancellation
        self._closed = True
        if self._response_stream is not None:
            try:
                self._response_stream.cancel()
            except UsageError:
                # doesn't matter if we're closed when...we're trying to close
                pass
            self._response_stream = None

    async def items(self):
        """
        Return an asynchronous stream of events.

        .. code-block:: python

            async with conn.query('SampleApp:Iou') as query:
                async for r in query:
                    print(f"Offset: {r.offset}")
                    for event in r.events:
                        print(f"  Event: {event}")

        :return:
            A stream of responses, where each response contains one or more events at a particular
            offset.

            At least one initial :class:`Boundary` is always returned, even if the stream is empty.
            In this case, the first returned object is a :class:`Boundary` with ``offset=None``.
        """
        log = self.conn.config.logger
        async with _translate_exceptions(self.conn), self, _allow_cancel(lambda: self._closed):
            # the token may change over time, but fix the "read_as" parties at the start,
            # even if those rights are changed over time
            read_as = await self._call.get_read_as()
            filters = await self.conn.codec.encode_filters(self._filters, token=self._call.token)
            filters_by_party = {party: filters for party in read_as}
            tx_filter_pb = lapipb.TransactionFilter(filters_by_party=filters_by_party)

            offset = self._offset_range.begin
            if offset:
                log.debug("Skipped reading from the ACS because begin offset is %r", offset)
            else:
                # when starting from the beginning of the ledger, the Active Contract Set service
                # lets us catch up more quickly than having to parse every create/archive event
                # ourselves
                log.debug("Reading from the ACS...")
                async for event in self._acs_events(tx_filter_pb):
                    if isinstance(event, CreateEvent):
                        await self._emit_create(event)
                    elif isinstance(event, Boundary):
                        offset = event.offset
                        await self._emit_boundary(event)
                    else:
                        warnings.warn(f"Received an unknown event: {event}", ProtocolWarning)
                    yield event

                # when reading from the Active Contract Set service, if we're supposed to stop
                # at "the end", then the Active Contract Set data is all that we'll return
                if self._offset_range.end == END:
                    log.debug(
                        "Not reading from transaction stream because we were only asked for a snapshot."
                    )
                    return

            # now start returning events as they come off the transaction stream; note this
            # stream will never naturally close, so it's on the caller to call close() or to
            # otherwise exit our current context
            log.debug("Reading a transaction stream: %s", self._offset_range)
            async for event in self._tx_events(tx_filter_pb, offset, self._offset_range.end):
                log.debug("Received an event: %s", event)
                if isinstance(event, CreateEvent):
                    await self._emit_create(event)
                elif isinstance(event, ArchiveEvent):
                    await self._emit_archive(event)
                elif isinstance(event, Boundary):
                    await self._emit_boundary(event)
                else:
                    warnings.warn(f"Received an unknown event: {event}", ProtocolWarning)
                yield event

    async def _acs_events(
        self, filter_pb: lapipb.TransactionFilter
    ) -> AsyncIterable[Union[CreateEvent, Boundary]]:
        stub = self._call.grpc_stub(lapipb.ActiveContractsServiceStub)
        request = lapipb.GetActiveContractsRequest(ledger_id=self._call.ledger_id, filter=filter_pb)
        self._response_stream = response_stream = stub.GetActiveContracts(
            request,
            **self._call.grpc_kwargs,
        )

        offset = None

        # Unidirectional gRPC streams cannot sensibly have a deadline because the stream may be
        # open indefinitely. However, if fetching an individual message from the stream takes a
        # long time here, we can reasonably assume that the stream is dead, because Active
        # Contract Set messages are really supposed to be sent as quickly as the server can send
        # them. In other words, a long timeout pause in the middle of pulling down ACS messages
        # would be highly unusual, so we treat them as fatal
        i = response_stream.__aiter__()
        while True:
            try:
                response = await wait_for(i.__anext__(), timeout=self._call.timeout_seconds)
            except StopAsyncIteration:
                break

            LOG.debug(
                "ACS start (offset %r, %d event(s))",
                response.offset,
                len(response.active_contracts),
            )
            for event in response.active_contracts:
                c_evt = await self.conn.codec.decode_created_event(event, token=self._call.token)
                if self._is_match(c_evt):
                    yield c_evt
            # for ActiveContractSetResponse messages, only the last offset is actually relevant
            LOG.debug("ACS end (offset %r)", response.offset)
            offset = response.offset
        yield Boundary(offset)

    async def _tx_events(
        self,
        filter_pb: lapipb.TransactionFilter,
        begin_offset: Optional[str],
        end_offset: Union[None, str, End],
    ) -> AsyncIterable[Union[CreateEvent, ArchiveEvent, Boundary]]:
        stub = self._call.grpc_stub(lapipb.TransactionServiceStub)

        last_offset = begin_offset

        while True:
            request = lapipb.GetTransactionsRequest(
                ledger_id=self._call.ledger_id,
                filter=filter_pb,
                begin=self.conn.codec.encode_begin_offset(last_offset),
                end=self.conn.codec.encode_end_offset(end_offset),
            )

            self._response_stream = response_stream = stub.GetTransactions(
                request,
                **self._call.grpc_kwargs,
            )
            i = response_stream.__aiter__()

            while True:
                try:
                    response = await wait_for(i.__anext__(), timeout=self._call.timeout_seconds)
                except StopAsyncIteration:
                    # a TransactionStream stopped with an EOF sent from the server. This means
                    # our work in reading this stream is done too.
                    return
                except asyncio.TimeoutError:
                    # There was an unusually large gap in time between transactions.
                    # Abort the current stream because we can't be sure whether the stream died
                    # or if the server is merely quiet. If the stream re-establishment fails,
                    # we have our answer.
                    #
                    # Note that this _SHOULD_ be transparent to our callers, because we only
                    # perform this check on a Transaction boundary, and if the stream resumes
                    # successfully, we pick up from the point where we disconnected.
                    LOG.debug("terminating a transaction stream because of inactivity")

                    try:
                        # clean up the old stream; we ignore errors since we're about to retry
                        # the connection anyway
                        response_stream.cancel()
                    except Exception:  # noqa
                        pass

                    # Perform a GetLedgerEnd request _purely_ to assert that the remote is healthy,
                    # because we can't reliably distinguish between a condition where transactions
                    # aren't streaming because of no activity, or because of an unresponsive server.
                    # If this operation fails, let the exception bubble up the stack.
                    await self.conn.get_ledger_end(
                        token=self._call.token, timeout=self._call.timeout
                    )

                    # abort the loop in which we read events from a stream, and start over again
                    # trying to establish a stream connection
                    break

                for tx in response.transactions:
                    for event in tx.events:
                        event_type = event.WhichOneof("event")
                        if event_type == "created":
                            c_evt = await self.conn.codec.decode_created_event(
                                event.created, token=self._call.token
                            )
                            if self._is_match(c_evt):
                                yield c_evt
                        elif event_type == "archived":
                            yield await self.conn.codec.decode_archived_event(event.archived)
                        else:
                            warnings.warn(f"Unknown Event({event_type}=...)", ProtocolWarning)
                    last_offset = tx.offset
                    yield Boundary(tx.offset)

    def _is_match(self, event: CreateEvent) -> bool:
        # if there are no filters, then everything is a match
        if self._filters is None:
            return True

        for name, f in self._filters.items():
            # for each filter, if this contract type could be interpreted as a match for the current
            # filter, then apply a client-side filter
            if is_match(name, event.contract_id.value_type):
                return f.client_side is None or f.client_side(event.payload)

        # TODO: We checked every name, but none of them matched. This means we received an event for an
        #  unexpected template name, and that could happen if the query was _interface_ based, as
        #  opposed to _template_ based. For now, return True so that interface-based queries work at all,
        #  but a better fix would be to check templates against a filter on the interface. This requires
        #  us to have a bit more intelligence about the types coming over the wire.
        return True



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from ....protocol.v30 import sequencing_pb2 as _sequencing_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class InitializeSequencerFromGenesisStateRequest(_message.Message):
    __slots__ = ("topology_snapshot", "domain_parameters")
    TOPOLOGY_SNAPSHOT_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    topology_snapshot: bytes
    domain_parameters: _sequencing_pb2.StaticDomainParameters
    def __init__(self, topology_snapshot: _Optional[bytes] = ..., domain_parameters: _Optional[_Union[_sequencing_pb2.StaticDomainParameters, _Mapping]] = ...) -> None: ...

class InitializeSequencerFromGenesisStateResponse(_message.Message):
    __slots__ = ("replicated",)
    REPLICATED_FIELD_NUMBER: _ClassVar[int]
    replicated: bool
    def __init__(self, replicated: bool = ...) -> None: ...

class InitializeSequencerFromOnboardingStateRequest(_message.Message):
    __slots__ = ("onboarding_state",)
    ONBOARDING_STATE_FIELD_NUMBER: _ClassVar[int]
    onboarding_state: bytes
    def __init__(self, onboarding_state: _Optional[bytes] = ...) -> None: ...

class InitializeSequencerFromOnboardingStateResponse(_message.Message):
    __slots__ = ("replicated",)
    REPLICATED_FIELD_NUMBER: _ClassVar[int]
    replicated: bool
    def __init__(self, replicated: bool = ...) -> None: ...



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: com/digitalasset/canton/domain/api/v30/sequencer_connect_service.proto
# Protobuf Python Version: 5.27.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    2,
    '',
    'com/digitalasset/canton/domain/api/v30/sequencer_connect_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ....protocol.v30 import sequencing_pb2 as com_dot_digitalasset_dot_canton_dot_protocol_dot_v30_dot_sequencing__pb2
from ....protocol.v30 import topology_pb2 as com_dot_digitalasset_dot_canton_dot_protocol_dot_v30_dot_topology__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nFcom/digitalasset/canton/domain/api/v30/sequencer_connect_service.proto\x12&com.digitalasset.canton.domain.api.v30\x1a\x35\x63om/digitalasset/canton/protocol/v30/sequencing.proto\x1a\x33\x63om/digitalasset/canton/protocol/v30/topology.proto\"\xee\x08\n\x10SequencerConnect\x1ax\n\x10HandshakeRequest\x12\x64\n\x11handshake_request\x18\x01 \x01(\x0b\x32\x37.com.digitalasset.canton.protocol.v30.Handshake.RequestR\x10handshakeRequest\x1a|\n\x11HandshakeResponse\x12g\n\x12handshake_response\x18\x01 \x01(\x0b\x32\x38.com.digitalasset.canton.protocol.v30.Handshake.ResponseR\x11handshakeResponse\x1a\x14\n\x12GetDomainIdRequest\x1aW\n\x13GetDomainIdResponse\x12\x1b\n\tdomain_id\x18\x01 \x01(\tR\x08\x64omainId\x12#\n\rsequencer_uid\x18\x02 \x01(\tR\x0csequencerUid\x1a\x1c\n\x1aGetDomainParametersRequest\x1a\x90\x01\n\x1bGetDomainParametersResponse\x12\x63\n\rparameters_v1\x18\x02 \x01(\x0b\x32<.com.digitalasset.canton.protocol.v30.StaticDomainParametersH\x00R\x0cparametersV1B\x0c\n\nparameters\x1a\x15\n\x13VerifyActiveRequest\x1a\xd0\x02\n\x14VerifyActiveResponse\x12q\n\x07success\x18\x01 \x01(\x0b\x32U.com.digitalasset.canton.domain.api.v30.SequencerConnect.VerifyActiveResponse.SuccessH\x00R\x07success\x12q\n\x07\x66\x61ilure\x18\x02 \x01(\x0b\x32U.com.digitalasset.canton.domain.api.v30.SequencerConnect.VerifyActiveResponse.FailureH\x00R\x07\x66\x61ilure\x1a&\n\x07Success\x12\x1b\n\tis_active\x18\x01 \x01(\x08R\x08isActive\x1a!\n\x07\x46\x61ilure\x12\x16\n\x06reason\x18\x01 \x01(\tR\x06reasonB\x07\n\x05value\x1a\xa5\x01\n-RegisterOnboardingTopologyTransactionsRequest\x12t\n\x15topology_transactions\x18\x01 \x03(\x0b\x32?.com.digitalasset.canton.protocol.v30.SignedTopologyTransactionR\x14topologyTransactions\x1a\x30\n.RegisterOnboardingTopologyTransactionsResponse2\xd6\x07\n\x17SequencerConnectService\x12\xa2\x01\n\tHandshake\x12I.com.digitalasset.canton.domain.api.v30.SequencerConnect.HandshakeRequest\x1aJ.com.digitalasset.canton.domain.api.v30.SequencerConnect.HandshakeResponse\x12\xa8\x01\n\x0bGetDomainId\x12K.com.digitalasset.canton.domain.api.v30.SequencerConnect.GetDomainIdRequest\x1aL.com.digitalasset.canton.domain.api.v30.SequencerConnect.GetDomainIdResponse\x12\xc0\x01\n\x13GetDomainParameters\x12S.com.digitalasset.canton.domain.api.v30.SequencerConnect.GetDomainParametersRequest\x1aT.com.digitalasset.canton.domain.api.v30.SequencerConnect.GetDomainParametersResponse\x12\xab\x01\n\x0cVerifyActive\x12L.com.digitalasset.canton.domain.api.v30.SequencerConnect.VerifyActiveRequest\x1aM.com.digitalasset.canton.domain.api.v30.SequencerConnect.VerifyActiveResponse\x12\xf9\x01\n&RegisterOnboardingTopologyTransactions\x12\x66.com.digitalasset.canton.domain.api.v30.SequencerConnect.RegisterOnboardingTopologyTransactionsRequest\x1ag.com.digitalasset.canton.domain.api.v30.SequencerConnect.RegisterOnboardingTopologyTransactionsResponseBWZUgithub.com/digital-asset/dazl-client/v8/go/api/com/digitalasset/canton/domain/api/v30b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'com.digitalasset.canton.domain.api.v30.sequencer_connect_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZUgithub.com/digital-asset/dazl-client/v8/go/api/com/digitalasset/canton/domain/api/v30'
  _globals['_SEQUENCERCONNECT']._serialized_start=223
  _globals['_SEQUENCERCONNECT']._serialized_end=1357
  _globals['_SEQUENCERCONNECT_HANDSHAKEREQUEST']._serialized_start=243
  _globals['_SEQUENCERCONNECT_HANDSHAKEREQUEST']._serialized_end=363
  _globals['_SEQUENCERCONNECT_HANDSHAKERESPONSE']._serialized_start=365
  _globals['_SEQUENCERCONNECT_HANDSHAKERESPONSE']._serialized_end=489
  _globals['_SEQUENCERCONNECT_GETDOMAINIDREQUEST']._serialized_start=491
  _globals['_SEQUENCERCONNECT_GETDOMAINIDREQUEST']._serialized_end=511
  _globals['_SEQUENCERCONNECT_GETDOMAINIDRESPONSE']._serialized_start=513
  _globals['_SEQUENCERCONNECT_GETDOMAINIDRESPONSE']._serialized_end=600
  _globals['_SEQUENCERCONNECT_GETDOMAINPARAMETERSREQUEST']._serialized_start=602
  _globals['_SEQUENCERCONNECT_GETDOMAINPARAMETERSREQUEST']._serialized_end=630
  _globals['_SEQUENCERCONNECT_GETDOMAINPARAMETERSRESPONSE']._serialized_start=633
  _globals['_SEQUENCERCONNECT_GETDOMAINPARAMETERSRESPONSE']._serialized_end=777
  _globals['_SEQUENCERCONNECT_VERIFYACTIVEREQUEST']._serialized_start=779
  _globals['_SEQUENCERCONNECT_VERIFYACTIVEREQUEST']._serialized_end=800
  _globals['_SEQUENCERCONNECT_VERIFYACTIVERESPONSE']._serialized_start=803
  _globals['_SEQUENCERCONNECT_VERIFYACTIVERESPONSE']._serialized_end=1139
  _globals['_SEQUENCERCONNECT_VERIFYACTIVERESPONSE_SUCCESS']._serialized_start=1057
  _globals['_SEQUENCERCONNECT_VERIFYACTIVERESPONSE_SUCCESS']._serialized_end=1095
  _globals['_SEQUENCERCONNECT_VERIFYACTIVERESPONSE_FAILURE']._serialized_start=1097
  _globals['_SEQUENCERCONNECT_VERIFYACTIVERESPONSE_FAILURE']._serialized_end=1130
  _globals['_SEQUENCERCONNECT_REGISTERONBOARDINGTOPOLOGYTRANSACTIONSREQUEST']._serialized_start=1142
  _globals['_SEQUENCERCONNECT_REGISTERONBOARDINGTOPOLOGYTRANSACTIONSREQUEST']._serialized_end=1307
  _globals['_SEQUENCERCONNECT_REGISTERONBOARDINGTOPOLOGYTRANSACTIONSRESPONSE']._serialized_start=1309
  _globals['_SEQUENCERCONNECT_REGISTERONBOARDINGTOPOLOGYTRANSACTIONSRESPONSE']._serialized_end=1357
  _globals['_SEQUENCERCONNECTSERVICE']._serialized_start=1360
  _globals['_SEQUENCERCONNECTSERVICE']._serialized_end=2342
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, typing as _typing

import grpc as _grpc
from grpc import aio as _grpc_aio

from .topology_manager_read_service_pb2 import ListAllRequest, ListAllResponse, ListAvailableStoresRequest, ListAvailableStoresResult, ListDomainParametersChangesRequest, ListDomainParametersChangesResult, ListIdentifierDelegationRequest, ListIdentifierDelegationResult, ListMediatorDomainStateRequest, ListMediatorDomainStateResult, ListNamespaceDelegationRequest, ListNamespaceDelegationResult, ListOwnerToKeyMappingRequest, ListOwnerToKeyMappingResult, ListParticipantDomainStateRequest, ListParticipantDomainStateResult, ListPartyToParticipantRequest, ListPartyToParticipantResult, ListSignedLegalIdentityClaimRequest, ListSignedLegalIdentityClaimResult, ListVettedPackagesRequest, ListVettedPackagesResult

__all__ = [
    "TopologyManagerReadServiceStub",
]


# noinspection PyPep8Naming,DuplicatedCode
class TopologyManagerReadServiceStub:
    @classmethod  # type: ignore
    @_typing.overload
    def __new__(cls, channel: _grpc.Channel) -> _TopologyManagerReadServiceBlockingStub: ...  # type: ignore
    @classmethod  # type: ignore
    @_typing.overload
    def __new__(cls, channel: _grpc_aio.Channel) -> _TopologyManagerReadServiceAsyncStub: ...  # type: ignore
    def ListAvailableStores(self, __1: ListAvailableStoresRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[ListAvailableStoresResult, _grpc_aio.UnaryUnaryCall[_typing.Any, ListAvailableStoresResult]]: ...
    def ListPartyToParticipant(self, __1: ListPartyToParticipantRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[ListPartyToParticipantResult, _grpc_aio.UnaryUnaryCall[_typing.Any, ListPartyToParticipantResult]]: ...
    def ListOwnerToKeyMapping(self, __1: ListOwnerToKeyMappingRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[ListOwnerToKeyMappingResult, _grpc_aio.UnaryUnaryCall[_typing.Any, ListOwnerToKeyMappingResult]]: ...
    def ListNamespaceDelegation(self, __1: ListNamespaceDelegationRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[ListNamespaceDelegationResult, _grpc_aio.UnaryUnaryCall[_typing.Any, ListNamespaceDelegationResult]]: ...
    def ListIdentifierDelegation(self, __1: ListIdentifierDelegationRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[ListIdentifierDelegationResult, _grpc_aio.UnaryUnaryCall[_typing.Any, ListIdentifierDelegationResult]]: ...
    def ListSignedLegalIdentityClaim(self, __1: ListSignedLegalIdentityClaimRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[ListSignedLegalIdentityClaimResult, _grpc_aio.UnaryUnaryCall[_typing.Any, ListSignedLegalIdentityClaimResult]]: ...
    def ListParticipantDomainState(self, __1: ListParticipantDomainStateRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[ListParticipantDomainStateResult, _grpc_aio.UnaryUnaryCall[_typing.Any, ListParticipantDomainStateResult]]: ...
    def ListMediatorDomainState(self, __1: ListMediatorDomainStateRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[ListMediatorDomainStateResult, _grpc_aio.UnaryUnaryCall[_typing.Any, ListMediatorDomainStateResult]]: ...
    def ListVettedPackages(self, __1: ListVettedPackagesRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[ListVettedPackagesResult, _grpc_aio.UnaryUnaryCall[_typing.Any, ListVettedPackagesResult]]: ...
    def ListDomainParametersChanges(self, __1: ListDomainParametersChangesRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[ListDomainParametersChangesResult, _grpc_aio.UnaryUnaryCall[_typing.Any, ListDomainParametersChangesResult]]: ...
    def ListAll(self, __1: ListAllRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _typing.Union[ListAllResponse, _grpc_aio.UnaryUnaryCall[_typing.Any, ListAllResponse]]: ...

# noinspection PyPep8Naming,DuplicatedCode
class _TopologyManagerReadServiceBlockingStub(TopologyManagerReadServiceStub):
    def ListAvailableStores(self, __1: ListAvailableStoresRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> ListAvailableStoresResult: ...
    def ListPartyToParticipant(self, __1: ListPartyToParticipantRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> ListPartyToParticipantResult: ...
    def ListOwnerToKeyMapping(self, __1: ListOwnerToKeyMappingRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> ListOwnerToKeyMappingResult: ...
    def ListNamespaceDelegation(self, __1: ListNamespaceDelegationRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> ListNamespaceDelegationResult: ...
    def ListIdentifierDelegation(self, __1: ListIdentifierDelegationRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> ListIdentifierDelegationResult: ...
    def ListSignedLegalIdentityClaim(self, __1: ListSignedLegalIdentityClaimRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> ListSignedLegalIdentityClaimResult: ...
    def ListParticipantDomainState(self, __1: ListParticipantDomainStateRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> ListParticipantDomainStateResult: ...
    def ListMediatorDomainState(self, __1: ListMediatorDomainStateRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> ListMediatorDomainStateResult: ...
    def ListVettedPackages(self, __1: ListVettedPackagesRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> ListVettedPackagesResult: ...
    def ListDomainParametersChanges(self, __1: ListDomainParametersChangesRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> ListDomainParametersChangesResult: ...
    def ListAll(self, __1: ListAllRequest, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_typing.Tuple[_typing.Tuple[str, _typing.Union[str, bytes]], ...]] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> ListAllResponse: ...

# noinspection PyPep8Naming,DuplicatedCode
class _TopologyManagerReadServiceAsyncStub(TopologyManagerReadServiceStub):
    def ListAvailableStores(self, __1: ListAvailableStoresRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, ListAvailableStoresResult]: ...  # type: ignore
    def ListPartyToParticipant(self, __1: ListPartyToParticipantRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, ListPartyToParticipantResult]: ...  # type: ignore
    def ListOwnerToKeyMapping(self, __1: ListOwnerToKeyMappingRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, ListOwnerToKeyMappingResult]: ...  # type: ignore
    def ListNamespaceDelegation(self, __1: ListNamespaceDelegationRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, ListNamespaceDelegationResult]: ...  # type: ignore
    def ListIdentifierDelegation(self, __1: ListIdentifierDelegationRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, ListIdentifierDelegationResult]: ...  # type: ignore
    def ListSignedLegalIdentityClaim(self, __1: ListSignedLegalIdentityClaimRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, ListSignedLegalIdentityClaimResult]: ...  # type: ignore
    def ListParticipantDomainState(self, __1: ListParticipantDomainStateRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, ListParticipantDomainStateResult]: ...  # type: ignore
    def ListMediatorDomainState(self, __1: ListMediatorDomainStateRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, ListMediatorDomainStateResult]: ...  # type: ignore
    def ListVettedPackages(self, __1: ListVettedPackagesRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, ListVettedPackagesResult]: ...  # type: ignore
    def ListDomainParametersChanges(self, __1: ListDomainParametersChangesRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, ListDomainParametersChangesResult]: ...  # type: ignore
    def ListAll(self, __1: ListAllRequest, *, timeout: _typing.Optional[float] = ..., metadata: _typing.Optional[_grpc_aio.Metadata] = ..., credentials: _typing.Optional[_grpc.CallCredentials] = ..., wait_for_ready: _typing.Optional[bool] = ..., compression: _typing.Optional[_grpc.Compression] = ...) -> _grpc_aio.UnaryUnaryCall[_typing.Any, ListAllResponse]: ...  # type: ignore



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from ...crypto.v0 import crypto_pb2 as _crypto_pb2
from . import causality_pb2 as _causality_pb2
from . import participant_transaction_pb2 as _participant_transaction_pb2
from . import participant_transfer_pb2 as _participant_transfer_pb2
from . import topology_pb2 as _topology_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class SignedProtocolMessage(_message.Message):
    __slots__ = ("signature", "mediator_response", "transaction_result", "malformed_mediator_request_result", "transfer_result", "acs_commitment")
    SIGNATURE_FIELD_NUMBER: _ClassVar[int]
    MEDIATOR_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_RESULT_FIELD_NUMBER: _ClassVar[int]
    MALFORMED_MEDIATOR_REQUEST_RESULT_FIELD_NUMBER: _ClassVar[int]
    TRANSFER_RESULT_FIELD_NUMBER: _ClassVar[int]
    ACS_COMMITMENT_FIELD_NUMBER: _ClassVar[int]
    signature: _crypto_pb2.Signature
    mediator_response: bytes
    transaction_result: bytes
    malformed_mediator_request_result: bytes
    transfer_result: bytes
    acs_commitment: bytes
    def __init__(self, signature: _Optional[_Union[_crypto_pb2.Signature, _Mapping]] = ..., mediator_response: _Optional[bytes] = ..., transaction_result: _Optional[bytes] = ..., malformed_mediator_request_result: _Optional[bytes] = ..., transfer_result: _Optional[bytes] = ..., acs_commitment: _Optional[bytes] = ...) -> None: ...

class EnvelopeContent(_message.Message):
    __slots__ = ("informee_message", "signed_message", "encrypted_view_message", "domain_topology_transaction_message", "transfer_out_mediator_message", "transfer_in_mediator_message", "root_hash_message", "register_topology_transaction_request", "register_topology_transaction_response", "causality_message")
    INFORMEE_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    SIGNED_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    ENCRYPTED_VIEW_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_TOPOLOGY_TRANSACTION_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    TRANSFER_OUT_MEDIATOR_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    TRANSFER_IN_MEDIATOR_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    ROOT_HASH_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    REGISTER_TOPOLOGY_TRANSACTION_REQUEST_FIELD_NUMBER: _ClassVar[int]
    REGISTER_TOPOLOGY_TRANSACTION_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    CAUSALITY_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    informee_message: _participant_transaction_pb2.InformeeMessage
    signed_message: SignedProtocolMessage
    encrypted_view_message: _participant_transaction_pb2.EncryptedViewMessage
    domain_topology_transaction_message: _topology_pb2.DomainTopologyTransactionMessage
    transfer_out_mediator_message: _participant_transfer_pb2.TransferOutMediatorMessage
    transfer_in_mediator_message: _participant_transfer_pb2.TransferInMediatorMessage
    root_hash_message: _participant_transaction_pb2.RootHashMessage
    register_topology_transaction_request: _topology_pb2.RegisterTopologyTransactionRequest
    register_topology_transaction_response: _topology_pb2.RegisterTopologyTransactionResponse
    causality_message: _causality_pb2.CausalityMessage
    def __init__(self, informee_message: _Optional[_Union[_participant_transaction_pb2.InformeeMessage, _Mapping]] = ..., signed_message: _Optional[_Union[SignedProtocolMessage, _Mapping]] = ..., encrypted_view_message: _Optional[_Union[_participant_transaction_pb2.EncryptedViewMessage, _Mapping]] = ..., domain_topology_transaction_message: _Optional[_Union[_topology_pb2.DomainTopologyTransactionMessage, _Mapping]] = ..., transfer_out_mediator_message: _Optional[_Union[_participant_transfer_pb2.TransferOutMediatorMessage, _Mapping]] = ..., transfer_in_mediator_message: _Optional[_Union[_participant_transfer_pb2.TransferInMediatorMessage, _Mapping]] = ..., root_hash_message: _Optional[_Union[_participant_transaction_pb2.RootHashMessage, _Mapping]] = ..., register_topology_transaction_request: _Optional[_Union[_topology_pb2.RegisterTopologyTransactionRequest, _Mapping]] = ..., register_topology_transaction_response: _Optional[_Union[_topology_pb2.RegisterTopologyTransactionResponse, _Mapping]] = ..., causality_message: _Optional[_Union[_causality_pb2.CausalityMessage, _Mapping]] = ...) -> None: ...



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: com/daml/ledger/api/v1/experimental_features.proto
# Protobuf Python Version: 5.27.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    2,
    '',
    'com/daml/ledger/api/v1/experimental_features.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2com/daml/ledger/api/v1/experimental_features.proto\x12\x16\x63om.daml.ledger.api.v1\"\x8d\n\n\x14\x45xperimentalFeatures\x12v\n\x18self_service_error_codes\x18\x01 \x01(\x0b\x32\x39.com.daml.ledger.api.v1.ExperimentalSelfServiceErrorCodesB\x02\x18\x01R\x15selfServiceErrorCodes\x12O\n\x0bstatic_time\x18\x02 \x01(\x0b\x32..com.daml.ledger.api.v1.ExperimentalStaticTimeR\nstaticTime\x12i\n\x15\x63ommand_deduplication\x18\x03 \x01(\x0b\x32\x34.com.daml.ledger.api.v1.CommandDeduplicationFeaturesR\x14\x63ommandDeduplication\x12\x62\n\x12optional_ledger_id\x18\x04 \x01(\x0b\x32\x34.com.daml.ledger.api.v1.ExperimentalOptionalLedgerIdR\x10optionalLedgerId\x12R\n\x0c\x63ontract_ids\x18\x05 \x01(\x0b\x32/.com.daml.ledger.api.v1.ExperimentalContractIdsR\x0b\x63ontractIds\x12\x65\n\x13\x63ommitter_event_log\x18\x06 \x01(\x0b\x32\x35.com.daml.ledger.api.v1.ExperimentalCommitterEventLogR\x11\x63ommitterEventLog\x12g\n\x13\x65xplicit_disclosure\x18\x07 \x01(\x0b\x32\x36.com.daml.ledger.api.v1.ExperimentalExplicitDisclosureR\x12\x65xplicitDisclosure\x12\x9e\x01\n(user_and_party_local_metadata_extensions\x18\x08 \x01(\x0b\x32G.com.daml.ledger.api.v1.ExperimentalUserAndPartyLocalMetadataExtensionsR#userAndPartyLocalMetadataExtensions\x12\x61\n\x14\x61\x63s_active_at_offset\x18\t \x01(\x0b\x32\x30.com.daml.ledger.api.v1.AcsActiveAtOffsetFeatureR\x11\x61\x63sActiveAtOffset\x12\x62\n\x10template_filters\x18\n \x01(\x0b\x32\x37.com.daml.ledger.api.v1.TransactionsWithTemplateFiltersR\x0ftemplateFilters\x12z\n\x1a\x63ommand_inspection_service\x18\x0b \x01(\x0b\x32<.com.daml.ledger.api.v1.ExperimentalCommandInspectionServiceR\x18\x63ommandInspectionService\x12U\n\x0epruned_offsets\x18\x0c \x01(\x0b\x32..com.daml.ledger.api.v1.StreamingPrunedOffsetsR\rprunedOffsets\"\'\n!ExperimentalSelfServiceErrorCodes:\x02\x18\x01\"6\n\x16\x45xperimentalStaticTime\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported\"\xcb\x02\n\x1c\x43ommandDeduplicationFeatures\x12{\n\x1c\x64\x65\x64uplication_period_support\x18\x01 \x01(\x0b\x32\x39.com.daml.ledger.api.v1.CommandDeduplicationPeriodSupportR\x1a\x64\x65\x64uplicationPeriodSupport\x12_\n\x12\x64\x65\x64uplication_type\x18\x02 \x01(\x0e\x32\x30.com.daml.ledger.api.v1.CommandDeduplicationTypeR\x11\x64\x65\x64uplicationType\x12M\n#max_deduplication_duration_enforced\x18\x03 \x01(\x08R maxDeduplicationDurationEnforced\"\x1e\n\x1c\x45xperimentalOptionalLedgerId\"\xbf\x03\n!CommandDeduplicationPeriodSupport\x12n\n\x0eoffset_support\x18\x01 \x01(\x0e\x32G.com.daml.ledger.api.v1.CommandDeduplicationPeriodSupport.OffsetSupportR\roffsetSupport\x12t\n\x10\x64uration_support\x18\x02 \x01(\x0e\x32I.com.daml.ledger.api.v1.CommandDeduplicationPeriodSupport.DurationSupportR\x0f\x64urationSupport\"d\n\rOffsetSupport\x12\x18\n\x14OFFSET_NOT_SUPPORTED\x10\x00\x12\x19\n\x15OFFSET_NATIVE_SUPPORT\x10\x01\x12\x1e\n\x1aOFFSET_CONVERT_TO_DURATION\x10\x02\"N\n\x0f\x44urationSupport\x12\x1b\n\x17\x44URATION_NATIVE_SUPPORT\x10\x00\x12\x1e\n\x1a\x44URATION_CONVERT_TO_OFFSET\x10\x01\"\xa5\x01\n\x17\x45xperimentalContractIds\x12S\n\x02v1\x18\x01 \x01(\x0e\x32\x43.com.daml.ledger.api.v1.ExperimentalContractIds.ContractIdV1SupportR\x02v1\"5\n\x13\x43ontractIdV1Support\x12\x0c\n\x08SUFFIXED\x10\x00\x12\x10\n\x0cNON_SUFFIXED\x10\x01\"\xcd\x01\n\x1d\x45xperimentalCommitterEventLog\x12q\n\x0e\x65vent_log_type\x18\x01 \x01(\x0e\x32K.com.daml.ledger.api.v1.ExperimentalCommitterEventLog.CommitterEventLogTypeR\x0c\x65ventLogType\"9\n\x15\x43ommitterEventLogType\x12\x0f\n\x0b\x43\x45NTRALIZED\x10\x00\x12\x0f\n\x0b\x44ISTRIBUTED\x10\x01\">\n\x1e\x45xperimentalExplicitDisclosure\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported\"O\n/ExperimentalUserAndPartyLocalMetadataExtensions\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported\"8\n\x18\x41\x63sActiveAtOffsetFeature\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported\"?\n\x1fTransactionsWithTemplateFilters\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported\"D\n$ExperimentalCommandInspectionService\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported\"6\n\x16StreamingPrunedOffsets\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported*I\n\x18\x43ommandDeduplicationType\x12\x0e\n\nASYNC_ONLY\x10\x00\x12\x1d\n\x19\x41SYNC_AND_CONCURRENT_SYNC\x10\x01\x42\x98\x01\n\x16\x63om.daml.ledger.api.v1B\x1e\x45xperimentalFeaturesOuterClassZEgithub.com/digital-asset/dazl-client/v8/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'com.daml.ledger.api.v1.experimental_features_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.daml.ledger.api.v1B\036ExperimentalFeaturesOuterClassZEgithub.com/digital-asset/dazl-client/v8/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _globals['_EXPERIMENTALFEATURES'].fields_by_name['self_service_error_codes']._loaded_options = None
  _globals['_EXPERIMENTALFEATURES'].fields_by_name['self_service_error_codes']._serialized_options = b'\030\001'
  _globals['_EXPERIMENTALSELFSERVICEERRORCODES']._loaded_options = None
  _globals['_EXPERIMENTALSELFSERVICEERRORCODES']._serialized_options = b'\030\001'
  _globals['_COMMANDDEDUPLICATIONTYPE']._serialized_start=3057
  _globals['_COMMANDDEDUPLICATIONTYPE']._serialized_end=3130
  _globals['_EXPERIMENTALFEATURES']._serialized_start=79
  _globals['_EXPERIMENTALFEATURES']._serialized_end=1372
  _globals['_EXPERIMENTALSELFSERVICEERRORCODES']._serialized_start=1374
  _globals['_EXPERIMENTALSELFSERVICEERRORCODES']._serialized_end=1413
  _globals['_EXPERIMENTALSTATICTIME']._serialized_start=1415
  _globals['_EXPERIMENTALSTATICTIME']._serialized_end=1469
  _globals['_COMMANDDEDUPLICATIONFEATURES']._serialized_start=1472
  _globals['_COMMANDDEDUPLICATIONFEATURES']._serialized_end=1803
  _globals['_EXPERIMENTALOPTIONALLEDGERID']._serialized_start=1805
  _globals['_EXPERIMENTALOPTIONALLEDGERID']._serialized_end=1835
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT']._serialized_start=1838
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT']._serialized_end=2285
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT_OFFSETSUPPORT']._serialized_start=2105
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT_OFFSETSUPPORT']._serialized_end=2205
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT_DURATIONSUPPORT']._serialized_start=2207
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT_DURATIONSUPPORT']._serialized_end=2285
  _globals['_EXPERIMENTALCONTRACTIDS']._serialized_start=2288
  _globals['_EXPERIMENTALCONTRACTIDS']._serialized_end=2453
  _globals['_EXPERIMENTALCONTRACTIDS_CONTRACTIDV1SUPPORT']._serialized_start=2400
  _globals['_EXPERIMENTALCONTRACTIDS_CONTRACTIDV1SUPPORT']._serialized_end=2453
  _globals['_EXPERIMENTALCOMMITTEREVENTLOG']._serialized_start=2456
  _globals['_EXPERIMENTALCOMMITTEREVENTLOG']._serialized_end=2661
  _globals['_EXPERIMENTALCOMMITTEREVENTLOG_COMMITTEREVENTLOGTYPE']._serialized_start=2604
  _globals['_EXPERIMENTALCOMMITTEREVENTLOG_COMMITTEREVENTLOGTYPE']._serialized_end=2661
  _globals['_EXPERIMENTALEXPLICITDISCLOSURE']._serialized_start=2663
  _globals['_EXPERIMENTALEXPLICITDISCLOSURE']._serialized_end=2725
  _globals['_EXPERIMENTALUSERANDPARTYLOCALMETADATAEXTENSIONS']._serialized_start=2727
  _globals['_EXPERIMENTALUSERANDPARTYLOCALMETADATAEXTENSIONS']._serialized_end=2806
  _globals['_ACSACTIVEATOFFSETFEATURE']._serialized_start=2808
  _globals['_ACSACTIVEATOFFSETFEATURE']._serialized_end=2864
  _globals['_TRANSACTIONSWITHTEMPLATEFILTERS']._serialized_start=2866
  _globals['_TRANSACTIONSWITHTEMPLATEFILTERS']._serialized_end=2929
  _globals['_EXPERIMENTALCOMMANDINSPECTIONSERVICE']._serialized_start=2931
  _globals['_EXPERIMENTALCOMMANDINSPECTIONSERVICE']._serialized_end=2999
  _globals['_STREAMINGPRUNEDOFFSETS']._serialized_start=3001
  _globals['_STREAMINGPRUNEDOFFSETS']._serialized_end=3055
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
"""
:mod:`dazl.ledger.blocking`
===========================

Thread-blocking protocols and base classes for connecting to a Daml ledger.

.. autoclass:: Connection()

.. autoclass:: QueryStream()
"""

from __future__ import annotations

from typing import Optional, Protocol, runtime_checkable

__all__ = ["PackageService", "Connection", "QueryStream"]

from ...damlast.daml_lf_1 import PackageRef
from ...prim import TimeDeltaLike
from ..auth import TokenOrTokenProvider


class PackageService(Protocol):
    """
    Protocol that describe a service that provides package information. The :class:`Connection`
    protocol extends this interface.
    """

    def get_package(
        self,
        package_id: PackageRef,
        /,
        *,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ) -> bytes:
        raise NotImplementedError

    def list_package_ids(
        self,
        *,
        token: Optional[TokenOrTokenProvider] = None,
        timeout: Optional[TimeDeltaLike] = None,
    ):
        raise NotImplementedError


@runtime_checkable
class Connection(Protocol):
    """
    Protocol that describes the interface for a blocking connection to a Daml ledger. The methods of
    an implementation should be thread-safe, with the exception of :meth:`open` and :meth:`close`.

    Since ``dazl`` activity is typically dominated by I/O, using a blocking :class:`Connection` with
    multiple threads is usually more expensive than using connections. You should consider
    using the asyncio variant of this API (:class:`dazl.ledger.aio.Connection`) instead.
    """

    def __enter__(self):
        raise NotImplementedError

    def __exit__(self, exc_type, exc_val, exc_tb):
        raise NotImplementedError


@runtime_checkable
class QueryStream(Protocol):
    """
    A blocking stream of offsets and events from a Daml ledger.
    """

    def __enter__(self):
        raise NotImplementedError

    def __exit__(self, exc_type, exc_val, exc_tb):
        raise NotImplementedError

    def __iter__(self):
        raise NotImplementedError



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from ....protocol.v0 import participant_transfer_pb2 as _participant_transfer_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AdminTransferOutRequest(_message.Message):
    __slots__ = ("submitting_party", "contract_id", "origin_domain", "target_domain", "application_id", "submission_id", "workflow_id", "command_id")
    SUBMITTING_PARTY_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_ID_FIELD_NUMBER: _ClassVar[int]
    ORIGIN_DOMAIN_FIELD_NUMBER: _ClassVar[int]
    TARGET_DOMAIN_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_ID_FIELD_NUMBER: _ClassVar[int]
    SUBMISSION_ID_FIELD_NUMBER: _ClassVar[int]
    WORKFLOW_ID_FIELD_NUMBER: _ClassVar[int]
    COMMAND_ID_FIELD_NUMBER: _ClassVar[int]
    submitting_party: str
    contract_id: str
    origin_domain: str
    target_domain: str
    application_id: str
    submission_id: str
    workflow_id: str
    command_id: str
    def __init__(self, submitting_party: _Optional[str] = ..., contract_id: _Optional[str] = ..., origin_domain: _Optional[str] = ..., target_domain: _Optional[str] = ..., application_id: _Optional[str] = ..., submission_id: _Optional[str] = ..., workflow_id: _Optional[str] = ..., command_id: _Optional[str] = ...) -> None: ...

class AdminTransferOutResponse(_message.Message):
    __slots__ = ("transfer_id",)
    TRANSFER_ID_FIELD_NUMBER: _ClassVar[int]
    transfer_id: _participant_transfer_pb2.TransferId
    def __init__(self, transfer_id: _Optional[_Union[_participant_transfer_pb2.TransferId, _Mapping]] = ...) -> None: ...

class AdminTransferInRequest(_message.Message):
    __slots__ = ("submitting_party_id", "target_domain", "transfer_id", "application_id", "submission_id", "workflow_id", "command_id")
    SUBMITTING_PARTY_ID_FIELD_NUMBER: _ClassVar[int]
    TARGET_DOMAIN_FIELD_NUMBER: _ClassVar[int]
    TRANSFER_ID_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_ID_FIELD_NUMBER: _ClassVar[int]
    SUBMISSION_ID_FIELD_NUMBER: _ClassVar[int]
    WORKFLOW_ID_FIELD_NUMBER: _ClassVar[int]
    COMMAND_ID_FIELD_NUMBER: _ClassVar[int]
    submitting_party_id: str
    target_domain: str
    transfer_id: _participant_transfer_pb2.TransferId
    application_id: str
    submission_id: str
    workflow_id: str
    command_id: str
    def __init__(self, submitting_party_id: _Optional[str] = ..., target_domain: _Optional[str] = ..., transfer_id: _Optional[_Union[_participant_transfer_pb2.TransferId, _Mapping]] = ..., application_id: _Optional[str] = ..., submission_id: _Optional[str] = ..., workflow_id: _Optional[str] = ..., command_id: _Optional[str] = ...) -> None: ...

class AdminTransferInResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class AdminTransferSearchQuery(_message.Message):
    __slots__ = ("search_domain", "filter_origin_domain", "filter_timestamp", "filter_submitting_party", "limit")
    SEARCH_DOMAIN_FIELD_NUMBER: _ClassVar[int]
    FILTER_ORIGIN_DOMAIN_FIELD_NUMBER: _ClassVar[int]
    FILTER_TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    FILTER_SUBMITTING_PARTY_FIELD_NUMBER: _ClassVar[int]
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    search_domain: str
    filter_origin_domain: str
    filter_timestamp: _timestamp_pb2.Timestamp
    filter_submitting_party: str
    limit: int
    def __init__(self, search_domain: _Optional[str] = ..., filter_origin_domain: _Optional[str] = ..., filter_timestamp: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., filter_submitting_party: _Optional[str] = ..., limit: _Optional[int] = ...) -> None: ...

class AdminTransferSearchResponse(_message.Message):
    __slots__ = ("results",)
    class TransferSearchResult(_message.Message):
        __slots__ = ("contract_id", "transfer_id", "origin_domain", "target_domain", "submitting_party", "ready_for_transfer_in", "target_time_proof")
        CONTRACT_ID_FIELD_NUMBER: _ClassVar[int]
        TRANSFER_ID_FIELD_NUMBER: _ClassVar[int]
        ORIGIN_DOMAIN_FIELD_NUMBER: _ClassVar[int]
        TARGET_DOMAIN_FIELD_NUMBER: _ClassVar[int]
        SUBMITTING_PARTY_FIELD_NUMBER: _ClassVar[int]
        READY_FOR_TRANSFER_IN_FIELD_NUMBER: _ClassVar[int]
        TARGET_TIME_PROOF_FIELD_NUMBER: _ClassVar[int]
        contract_id: str
        transfer_id: _participant_transfer_pb2.TransferId
        origin_domain: str
        target_domain: str
        submitting_party: str
        ready_for_transfer_in: bool
        target_time_proof: _timestamp_pb2.Timestamp
        def __init__(self, contract_id: _Optional[str] = ..., transfer_id: _Optional[_Union[_participant_transfer_pb2.TransferId, _Mapping]] = ..., origin_domain: _Optional[str] = ..., target_domain: _Optional[str] = ..., submitting_party: _Optional[str] = ..., ready_for_transfer_in: bool = ..., target_time_proof: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    results: _containers.RepeatedCompositeFieldContainer[AdminTransferSearchResponse.TransferSearchResult]
    def __init__(self, results: _Optional[_Iterable[_Union[AdminTransferSearchResponse.TransferSearchResult, _Mapping]]] = ...) -> None: ...



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from ....crypto.v30 import crypto_pb2 as _crypto_pb2
from ....protocol.v30 import common_stable_pb2 as _common_stable_pb2
from ....protocol.v30 import traffic_control_parameters_pb2 as _traffic_control_parameters_pb2
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class SequencerSnapshot(_message.Message):
    __slots__ = ("latest_timestamp", "last_block_height", "head_member_counters", "status", "additional", "in_flight_aggregations", "traffic_purchased", "traffic_consumed")
    class MemberCounter(_message.Message):
        __slots__ = ("member", "sequencer_counter")
        MEMBER_FIELD_NUMBER: _ClassVar[int]
        SEQUENCER_COUNTER_FIELD_NUMBER: _ClassVar[int]
        member: str
        sequencer_counter: int
        def __init__(self, member: _Optional[str] = ..., sequencer_counter: _Optional[int] = ...) -> None: ...
    class InFlightAggregationWithId(_message.Message):
        __slots__ = ("aggregation_id", "aggregation_rule", "max_sequencing_time", "aggregated_senders")
        AGGREGATION_ID_FIELD_NUMBER: _ClassVar[int]
        AGGREGATION_RULE_FIELD_NUMBER: _ClassVar[int]
        MAX_SEQUENCING_TIME_FIELD_NUMBER: _ClassVar[int]
        AGGREGATED_SENDERS_FIELD_NUMBER: _ClassVar[int]
        aggregation_id: bytes
        aggregation_rule: _common_stable_pb2.AggregationRule
        max_sequencing_time: int
        aggregated_senders: _containers.RepeatedCompositeFieldContainer[SequencerSnapshot.AggregationBySender]
        def __init__(self, aggregation_id: _Optional[bytes] = ..., aggregation_rule: _Optional[_Union[_common_stable_pb2.AggregationRule, _Mapping]] = ..., max_sequencing_time: _Optional[int] = ..., aggregated_senders: _Optional[_Iterable[_Union[SequencerSnapshot.AggregationBySender, _Mapping]]] = ...) -> None: ...
    class AggregationBySender(_message.Message):
        __slots__ = ("sender", "sequencing_timestamp", "signatures_by_envelope")
        SENDER_FIELD_NUMBER: _ClassVar[int]
        SEQUENCING_TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
        SIGNATURES_BY_ENVELOPE_FIELD_NUMBER: _ClassVar[int]
        sender: str
        sequencing_timestamp: int
        signatures_by_envelope: _containers.RepeatedCompositeFieldContainer[SequencerSnapshot.SignaturesForEnvelope]
        def __init__(self, sender: _Optional[str] = ..., sequencing_timestamp: _Optional[int] = ..., signatures_by_envelope: _Optional[_Iterable[_Union[SequencerSnapshot.SignaturesForEnvelope, _Mapping]]] = ...) -> None: ...
    class SignaturesForEnvelope(_message.Message):
        __slots__ = ("signatures",)
        SIGNATURES_FIELD_NUMBER: _ClassVar[int]
        signatures: _containers.RepeatedCompositeFieldContainer[_crypto_pb2.Signature]
        def __init__(self, signatures: _Optional[_Iterable[_Union[_crypto_pb2.Signature, _Mapping]]] = ...) -> None: ...
    LATEST_TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    LAST_BLOCK_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    HEAD_MEMBER_COUNTERS_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    ADDITIONAL_FIELD_NUMBER: _ClassVar[int]
    IN_FLIGHT_AGGREGATIONS_FIELD_NUMBER: _ClassVar[int]
    TRAFFIC_PURCHASED_FIELD_NUMBER: _ClassVar[int]
    TRAFFIC_CONSUMED_FIELD_NUMBER: _ClassVar[int]
    latest_timestamp: int
    last_block_height: int
    head_member_counters: _containers.RepeatedCompositeFieldContainer[SequencerSnapshot.MemberCounter]
    status: SequencerPruningStatus
    additional: ImplementationSpecificInfo
    in_flight_aggregations: _containers.RepeatedCompositeFieldContainer[SequencerSnapshot.InFlightAggregationWithId]
    traffic_purchased: _containers.RepeatedCompositeFieldContainer[_traffic_control_parameters_pb2.TrafficPurchased]
    traffic_consumed: _containers.RepeatedCompositeFieldContainer[_traffic_control_parameters_pb2.TrafficConsumed]
    def __init__(self, latest_timestamp: _Optional[int] = ..., last_block_height: _Optional[int] = ..., head_member_counters: _Optional[_Iterable[_Union[SequencerSnapshot.MemberCounter, _Mapping]]] = ..., status: _Optional[_Union[SequencerPruningStatus, _Mapping]] = ..., additional: _Optional[_Union[ImplementationSpecificInfo, _Mapping]] = ..., in_flight_aggregations: _Optional[_Iterable[_Union[SequencerSnapshot.InFlightAggregationWithId, _Mapping]]] = ..., traffic_purchased: _Optional[_Iterable[_Union[_traffic_control_parameters_pb2.TrafficPurchased, _Mapping]]] = ..., traffic_consumed: _Optional[_Iterable[_Union[_traffic_control_parameters_pb2.TrafficConsumed, _Mapping]]] = ...) -> None: ...

class SequencerMemberStatus(_message.Message):
    __slots__ = ("member", "registered_at", "last_acknowledged", "enabled")
    MEMBER_FIELD_NUMBER: _ClassVar[int]
    REGISTERED_AT_FIELD_NUMBER: _ClassVar[int]
    LAST_ACKNOWLEDGED_FIELD_NUMBER: _ClassVar[int]
    ENABLED_FIELD_NUMBER: _ClassVar[int]
    member: str
    registered_at: int
    last_acknowledged: _wrappers_pb2.Int64Value
    enabled: bool
    def __init__(self, member: _Optional[str] = ..., registered_at: _Optional[int] = ..., last_acknowledged: _Optional[_Union[_wrappers_pb2.Int64Value, _Mapping]] = ..., enabled: bool = ...) -> None: ...

class SequencerPruningStatus(_message.Message):
    __slots__ = ("now", "earliest_event_timestamp", "members")
    NOW_FIELD_NUMBER: _ClassVar[int]
    EARLIEST_EVENT_TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    MEMBERS_FIELD_NUMBER: _ClassVar[int]
    now: int
    earliest_event_timestamp: int
    members: _containers.RepeatedCompositeFieldContainer[SequencerMemberStatus]
    def __init__(self, now: _Optional[int] = ..., earliest_event_timestamp: _Optional[int] = ..., members: _Optional[_Iterable[_Union[SequencerMemberStatus, _Mapping]]] = ...) -> None: ...

class ImplementationSpecificInfo(_message.Message):
    __slots__ = ("implementation_name", "info")
    IMPLEMENTATION_NAME_FIELD_NUMBER: _ClassVar[int]
    INFO_FIELD_NUMBER: _ClassVar[int]
    implementation_name: str
    info: bytes
    def __init__(self, implementation_name: _Optional[str] = ..., info: _Optional[bytes] = ...) -> None: ...



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from . import domain_connectivity_pb2 as _domain_connectivity_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class DownloadRequest(_message.Message):
    __slots__ = ("parties", "filter_domain_id", "timestamp", "protocol_version", "chunkSize", "gzip_format", "contract_domain_renames", "parties_offboarding")
    class ContractDomainRenamesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    PARTIES_FIELD_NUMBER: _ClassVar[int]
    FILTER_DOMAIN_ID_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    PROTOCOL_VERSION_FIELD_NUMBER: _ClassVar[int]
    CHUNKSIZE_FIELD_NUMBER: _ClassVar[int]
    GZIP_FORMAT_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_DOMAIN_RENAMES_FIELD_NUMBER: _ClassVar[int]
    PARTIES_OFFBOARDING_FIELD_NUMBER: _ClassVar[int]
    parties: _containers.RepeatedScalarFieldContainer[str]
    filter_domain_id: str
    timestamp: _timestamp_pb2.Timestamp
    protocol_version: str
    chunkSize: _wrappers_pb2.UInt32Value
    gzip_format: bool
    contract_domain_renames: _containers.ScalarMap[str, str]
    parties_offboarding: bool
    def __init__(self, parties: _Optional[_Iterable[str]] = ..., filter_domain_id: _Optional[str] = ..., timestamp: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., protocol_version: _Optional[str] = ..., chunkSize: _Optional[_Union[_wrappers_pb2.UInt32Value, _Mapping]] = ..., gzip_format: bool = ..., contract_domain_renames: _Optional[_Mapping[str, str]] = ..., parties_offboarding: bool = ...) -> None: ...

class PurgeContractsRequest(_message.Message):
    __slots__ = ("domain", "contract_ids", "ignore_already_purged", "offboarded_parties")
    DOMAIN_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_IDS_FIELD_NUMBER: _ClassVar[int]
    IGNORE_ALREADY_PURGED_FIELD_NUMBER: _ClassVar[int]
    OFFBOARDED_PARTIES_FIELD_NUMBER: _ClassVar[int]
    domain: str
    contract_ids: _containers.RepeatedScalarFieldContainer[str]
    ignore_already_purged: bool
    offboarded_parties: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, domain: _Optional[str] = ..., contract_ids: _Optional[_Iterable[str]] = ..., ignore_already_purged: bool = ..., offboarded_parties: _Optional[_Iterable[str]] = ...) -> None: ...

class PurgeContractsResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class AcsSnapshotChunk(_message.Message):
    __slots__ = ("chunk",)
    CHUNK_FIELD_NUMBER: _ClassVar[int]
    chunk: bytes
    def __init__(self, chunk: _Optional[bytes] = ...) -> None: ...

class UploadRequest(_message.Message):
    __slots__ = ("acs_snapshot", "gzip_format")
    ACS_SNAPSHOT_FIELD_NUMBER: _ClassVar[int]
    GZIP_FORMAT_FIELD_NUMBER: _ClassVar[int]
    acs_snapshot: bytes
    gzip_format: bool
    def __init__(self, acs_snapshot: _Optional[bytes] = ..., gzip_format: bool = ...) -> None: ...

class UploadResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MigrateDomainRequest(_message.Message):
    __slots__ = ("source_alias", "target_domain_connection_config", "force")
    SOURCE_ALIAS_FIELD_NUMBER: _ClassVar[int]
    TARGET_DOMAIN_CONNECTION_CONFIG_FIELD_NUMBER: _ClassVar[int]
    FORCE_FIELD_NUMBER: _ClassVar[int]
    source_alias: str
    target_domain_connection_config: _domain_connectivity_pb2.DomainConnectionConfig
    force: _wrappers_pb2.BoolValue
    def __init__(self, source_alias: _Optional[str] = ..., target_domain_connection_config: _Optional[_Union[_domain_connectivity_pb2.DomainConnectionConfig, _Mapping]] = ..., force: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...) -> None: ...

class MigrateDomainResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ActiveContract(_message.Message):
    __slots__ = ("protocol_version", "domain_id", "contract")
    PROTOCOL_VERSION_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_ID_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_FIELD_NUMBER: _ClassVar[int]
    protocol_version: int
    domain_id: str
    contract: bytes
    def __init__(self, protocol_version: _Optional[int] = ..., domain_id: _Optional[str] = ..., contract: _Optional[bytes] = ...) -> None: ...

class ExportAcsRequest(_message.Message):
    __slots__ = ("parties", "filter_domain_id", "timestamp", "contract_domain_renames", "parties_offboarding")
    class TargetDomain(_message.Message):
        __slots__ = ("domain_id", "protocol_version")
        DOMAIN_ID_FIELD_NUMBER: _ClassVar[int]
        PROTOCOL_VERSION_FIELD_NUMBER: _ClassVar[int]
        domain_id: str
        protocol_version: int
        def __init__(self, domain_id: _Optional[str] = ..., protocol_version: _Optional[int] = ...) -> None: ...
    class ContractDomainRenamesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: ExportAcsRequest.TargetDomain
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[ExportAcsRequest.TargetDomain, _Mapping]] = ...) -> None: ...
    PARTIES_FIELD_NUMBER: _ClassVar[int]
    FILTER_DOMAIN_ID_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_DOMAIN_RENAMES_FIELD_NUMBER: _ClassVar[int]
    PARTIES_OFFBOARDING_FIELD_NUMBER: _ClassVar[int]
    parties: _containers.RepeatedScalarFieldContainer[str]
    filter_domain_id: str
    timestamp: _timestamp_pb2.Timestamp
    contract_domain_renames: _containers.MessageMap[str, ExportAcsRequest.TargetDomain]
    parties_offboarding: bool
    def __init__(self, parties: _Optional[_Iterable[str]] = ..., filter_domain_id: _Optional[str] = ..., timestamp: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., contract_domain_renames: _Optional[_Mapping[str, ExportAcsRequest.TargetDomain]] = ..., parties_offboarding: bool = ...) -> None: ...

class ExportAcsResponse(_message.Message):
    __slots__ = ("chunk",)
    CHUNK_FIELD_NUMBER: _ClassVar[int]
    chunk: bytes
    def __init__(self, chunk: _Optional[bytes] = ...) -> None: ...

class ImportAcsRequest(_message.Message):
    __slots__ = ("acs_snapshot", "workflow_id_prefix", "onboarded_parties")
    ACS_SNAPSHOT_FIELD_NUMBER: _ClassVar[int]
    WORKFLOW_ID_PREFIX_FIELD_NUMBER: _ClassVar[int]
    ONBOARDED_PARTIES_FIELD_NUMBER: _ClassVar[int]
    acs_snapshot: bytes
    workflow_id_prefix: str
    onboarded_parties: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, acs_snapshot: _Optional[bytes] = ..., workflow_id_prefix: _Optional[str] = ..., onboarded_parties: _Optional[_Iterable[str]] = ...) -> None: ...

class ImportAcsResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class PurgeDeactivatedDomainRequest(_message.Message):
    __slots__ = ("domain_alias",)
    DOMAIN_ALIAS_FIELD_NUMBER: _ClassVar[int]
    domain_alias: str
    def __init__(self, domain_alias: _Optional[str] = ...) -> None: ...

class PurgeDeactivatedDomainResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class IgnoreEventsRequest(_message.Message):
    __slots__ = ("domain_id", "from_inclusive", "to_inclusive", "force")
    DOMAIN_ID_FIELD_NUMBER: _ClassVar[int]
    FROM_INCLUSIVE_FIELD_NUMBER: _ClassVar[int]
    TO_INCLUSIVE_FIELD_NUMBER: _ClassVar[int]
    FORCE_FIELD_NUMBER: _ClassVar[int]
    domain_id: str
    from_inclusive: int
    to_inclusive: int
    force: bool
    def __init__(self, domain_id: _Optional[str] = ..., from_inclusive: _Optional[int] = ..., to_inclusive: _Optional[int] = ..., force: bool = ...) -> None: ...

class IgnoreEventsResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class UnignoreEventsRequest(_message.Message):
    __slots__ = ("domain_id", "from_inclusive", "to_inclusive", "force")
    DOMAIN_ID_FIELD_NUMBER: _ClassVar[int]
    FROM_INCLUSIVE_FIELD_NUMBER: _ClassVar[int]
    TO_INCLUSIVE_FIELD_NUMBER: _ClassVar[int]
    FORCE_FIELD_NUMBER: _ClassVar[int]
    domain_id: str
    from_inclusive: int
    to_inclusive: int
    force: bool
    def __init__(self, domain_id: _Optional[str] = ..., from_inclusive: _Optional[int] = ..., to_inclusive: _Optional[int] = ..., force: bool = ...) -> None: ...

class UnignoreEventsResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from ...crypto.v30 import crypto_pb2 as _crypto_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ActiveContract(_message.Message):
    __slots__ = ("domain_id", "contract", "reassignment_counter")
    DOMAIN_ID_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_FIELD_NUMBER: _ClassVar[int]
    REASSIGNMENT_COUNTER_FIELD_NUMBER: _ClassVar[int]
    domain_id: str
    contract: Contract
    reassignment_counter: int
    def __init__(self, domain_id: _Optional[str] = ..., contract: _Optional[_Union[Contract, _Mapping]] = ..., reassignment_counter: _Optional[int] = ...) -> None: ...

class Contract(_message.Message):
    __slots__ = ("contract_id", "raw_contract_instance", "metadata", "ledger_create_time", "contract_salt")
    class Metadata(_message.Message):
        __slots__ = ("non_maintainer_signatories", "non_signatory_stakeholders", "key", "maintainers")
        NON_MAINTAINER_SIGNATORIES_FIELD_NUMBER: _ClassVar[int]
        NON_SIGNATORY_STAKEHOLDERS_FIELD_NUMBER: _ClassVar[int]
        KEY_FIELD_NUMBER: _ClassVar[int]
        MAINTAINERS_FIELD_NUMBER: _ClassVar[int]
        non_maintainer_signatories: _containers.RepeatedScalarFieldContainer[str]
        non_signatory_stakeholders: _containers.RepeatedScalarFieldContainer[str]
        key: Contract.GlobalKey
        maintainers: _containers.RepeatedScalarFieldContainer[str]
        def __init__(self, non_maintainer_signatories: _Optional[_Iterable[str]] = ..., non_signatory_stakeholders: _Optional[_Iterable[str]] = ..., key: _Optional[_Union[Contract.GlobalKey, _Mapping]] = ..., maintainers: _Optional[_Iterable[str]] = ...) -> None: ...
    class GlobalKey(_message.Message):
        __slots__ = ("template_id", "key", "package_name")
        TEMPLATE_ID_FIELD_NUMBER: _ClassVar[int]
        KEY_FIELD_NUMBER: _ClassVar[int]
        PACKAGE_NAME_FIELD_NUMBER: _ClassVar[int]
        template_id: bytes
        key: bytes
        package_name: str
        def __init__(self, template_id: _Optional[bytes] = ..., key: _Optional[bytes] = ..., package_name: _Optional[str] = ...) -> None: ...
    CONTRACT_ID_FIELD_NUMBER: _ClassVar[int]
    RAW_CONTRACT_INSTANCE_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    LEDGER_CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_SALT_FIELD_NUMBER: _ClassVar[int]
    contract_id: str
    raw_contract_instance: bytes
    metadata: Contract.Metadata
    ledger_create_time: _timestamp_pb2.Timestamp
    contract_salt: _crypto_pb2.Salt
    def __init__(self, contract_id: _Optional[str] = ..., raw_contract_instance: _Optional[bytes] = ..., metadata: _Optional[_Union[Contract.Metadata, _Mapping]] = ..., ledger_create_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., contract_salt: _Optional[_Union[_crypto_pb2.Salt, _Mapping]] = ...) -> None: ...



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: com/digitalasset/canton/domain/protocol/v30/aggregation.proto
# Protobuf Python Version: 5.27.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    2,
    '',
    'com/digitalasset/canton/domain/protocol/v30/aggregation.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ....crypto.v30 import crypto_pb2 as com_dot_digitalasset_dot_canton_dot_crypto_dot_v30_dot_crypto__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n=com/digitalasset/canton/domain/protocol/v30/aggregation.proto\x12+com.digitalasset.canton.domain.protocol.v30\x1a/com/digitalasset/canton/crypto/v30/crypto.proto\"\x9e\x02\n\x1c\x41ggregatedSignaturesOfSender\x12\x95\x01\n\x16signatures_by_envelope\x18\x01 \x03(\x0b\x32_.com.digitalasset.canton.domain.protocol.v30.AggregatedSignaturesOfSender.SignaturesForEnvelopeR\x14signaturesByEnvelope\x1a\x66\n\x15SignaturesForEnvelope\x12M\n\nsignatures\x18\x03 \x03(\x0b\x32-.com.digitalasset.canton.crypto.v30.SignatureR\nsignaturesB\\ZZgithub.com/digital-asset/dazl-client/v8/go/api/com/digitalasset/canton/domain/protocol/v30b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'com.digitalasset.canton.domain.protocol.v30.aggregation_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZZgithub.com/digital-asset/dazl-client/v8/go/api/com/digitalasset/canton/domain/protocol/v30'
  _globals['_AGGREGATEDSIGNATURESOFSENDER']._serialized_start=160
  _globals['_AGGREGATEDSIGNATURESOFSENDER']._serialized_end=446
  _globals['_AGGREGATEDSIGNATURESOFSENDER_SIGNATURESFORENVELOPE']._serialized_start=344
  _globals['_AGGREGATEDSIGNATURESOFSENDER_SIGNATURESFORENVELOPE']._serialized_end=446
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: com/digitalasset/canton/domain/admin/v0/sequencer_initialization_snapshot.proto
# Protobuf Python Version: 5.27.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    2,
    '',
    'com/digitalasset/canton/domain/admin/v0/sequencer_initialization_snapshot.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import sequencer_administration_service_pb2 as com_dot_digitalasset_dot_canton_dot_domain_dot_admin_dot_v0_dot_sequencer__administration__service__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nOcom/digitalasset/canton/domain/admin/v0/sequencer_initialization_snapshot.proto\x12\'com.digitalasset.canton.domain.admin.v0\x1aNcom/digitalasset/canton/domain/admin/v0/sequencer_administration_service.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xe6\x03\n\x11SequencerSnapshot\x12\x45\n\x10latest_timestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0flatestTimestamp\x12\x84\x01\n\x14head_member_counters\x18\x02 \x03(\x0b\x32R.com.digitalasset.canton.domain.admin.v0.SequencerSnapshot.HeadMemberCountersEntryR\x12headMemberCounters\x12W\n\x06status\x18\x03 \x01(\x0b\x32?.com.digitalasset.canton.domain.admin.v0.SequencerPruningStatusR\x06status\x12\x63\n\nadditional\x18\x04 \x01(\x0b\x32\x43.com.digitalasset.canton.domain.admin.v0.ImplementationSpecificInfoR\nadditional\x1a\x45\n\x17HeadMemberCountersEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\x03R\x05value:\x02\x38\x01\"a\n\x1aImplementationSpecificInfo\x12/\n\x13implementation_name\x18\x01 \x01(\tR\x12implementationName\x12\x12\n\x04info\x18\x02 \x01(\x0cR\x04infoBXZVgithub.com/digital-asset/dazl-client/v8/go/api/com/digitalasset/canton/domain/admin/v0b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'com.digitalasset.canton.domain.admin.v0.sequencer_initialization_snapshot_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZVgithub.com/digital-asset/dazl-client/v8/go/api/com/digitalasset/canton/domain/admin/v0'
  _globals['_SEQUENCERSNAPSHOT_HEADMEMBERCOUNTERSENTRY']._loaded_options = None
  _globals['_SEQUENCERSNAPSHOT_HEADMEMBERCOUNTERSENTRY']._serialized_options = b'8\001'
  _globals['_SEQUENCERSNAPSHOT']._serialized_start=238
  _globals['_SEQUENCERSNAPSHOT']._serialized_end=724
  _globals['_SEQUENCERSNAPSHOT_HEADMEMBERCOUNTERSENTRY']._serialized_start=655
  _globals['_SEQUENCERSNAPSHOT_HEADMEMBERCOUNTERSENTRY']._serialized_end=724
  _globals['_IMPLEMENTATIONSPECIFICINFO']._serialized_start=726
  _globals['_IMPLEMENTATIONSPECIFICINFO']._serialized_end=823
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class FetchTimeRequest(_message.Message):
    __slots__ = ("domain_id", "freshness_bound")
    DOMAIN_ID_FIELD_NUMBER: _ClassVar[int]
    FRESHNESS_BOUND_FIELD_NUMBER: _ClassVar[int]
    domain_id: _wrappers_pb2.StringValue
    freshness_bound: _duration_pb2.Duration
    def __init__(self, domain_id: _Optional[_Union[_wrappers_pb2.StringValue, _Mapping]] = ..., freshness_bound: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ...) -> None: ...

class FetchTimeResponse(_message.Message):
    __slots__ = ("timestamp",)
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    timestamp: _timestamp_pb2.Timestamp
    def __init__(self, timestamp: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class AwaitTimeRequest(_message.Message):
    __slots__ = ("domain_id", "timestamp")
    DOMAIN_ID_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    domain_id: _wrappers_pb2.StringValue
    timestamp: _timestamp_pb2.Timestamp
    def __init__(self, domain_id: _Optional[_Union[_wrappers_pb2.StringValue, _Mapping]] = ..., timestamp: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class SetPassive(_message.Message):
    __slots__ = ()
    class Request(_message.Message):
        __slots__ = ()
        def __init__(self) -> None: ...
    class Response(_message.Message):
        __slots__ = ("became_passive",)
        BECAME_PASSIVE_FIELD_NUMBER: _ClassVar[int]
        became_passive: bool
        def __init__(self, became_passive: bool = ...) -> None: ...
    def __init__(self) -> None: ...



# Copyright (c) 2017-2025 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from os.path import splitext
from pathlib import Path
import shutil

from google.protobuf.descriptor_pb2 import FileDescriptorSet

from .. import protoc
from ..syntax.python import rewrite_file_content
from .python_grpc_pyi import write_grpc_pyi_files
from .python_header import HEADER
from .python_init import write_init_files
from .util import get_root_name

__all__ = ["python_files"]


def python_files(from_: Path, to: Path) -> None:
    fds = FileDescriptorSet()
    fds.ParseFromString(from_.read_bytes())

    if to.exists():
        shutil.rmtree(to)

    write_init_files(fds, to)
    write_grpc_pyi_files(fds, to)
    write_standard_files(from_, fds, to)


def write_standard_files(from_: Path, fds: FileDescriptorSet, to: Path) -> None:
    """
    Writes (most of) the files from the standard Protobuf plugins.

    Files that came from the gRPC plugin but correspond to Protobufs that have no services are omitted.
    """
    response = protoc.run(from_, plugins=["python", "pyi", "grpc_python"])
    proto_files = {splitext(fd.name)[0]: fd for fd in fds.file}

    for f in response.file:
        root_name, suffix = get_root_name(f.name)
        if suffix == "_pb2_grpc.py" and not proto_files[root_name].service:
            # skip gRPC generated files if the corresponding Protobuf has no services
            continue

        p = to / f.name
        p.parent.mkdir(parents=True, exist_ok=True)

        # add a copyright to each of the files from the plugin
        p.write_text(HEADER + rewrite_file_content(f.name, f.content))
